@@ -5,20 +5,22 @@ import click
 
 from ..core.instance import DagsterInstance
 from ..version import __version__
+from .api import api_cli
+from .asset import asset_cli
 from .instance import instance_cli
 from .pipeline import pipeline_cli
-from .repository import repository_cli
 from .run import run_cli
 from .schedule import schedule_cli
 
 
 def create_dagster_cli():
     commands = {
+        'api': api_cli,
         'pipeline': pipeline_cli,
-        'repository': repository_cli,
         'run': run_cli,
         'instance': instance_cli,
         'schedule': schedule_cli,
+        'asset': asset_cli,
     }
 
     @click.group(commands=commands)
