@@ -28,7 +28,11 @@ class ConfigTypeKind(PythonEnum):
     @staticmethod
     def is_closed_generic(kind):
         check.inst_param(kind, 'kind', ConfigTypeKind)
-        return kind == ConfigTypeKind.ARRAY or kind == ConfigTypeKind.NONEABLE
+        return (
+            kind == ConfigTypeKind.ARRAY
+            or kind == ConfigTypeKind.NONEABLE
+            or kind == ConfigTypeKind.SCALAR_UNION
+        )
 
     @staticmethod
     def is_shape(kind):
@@ -64,58 +68,64 @@ class ConfigType(object):
         check.invariant(BuiltinEnum.contains(builtin_enum), 'param must be member of BuiltinEnum')
         return _CONFIG_MAP[builtin_enum]
 
-    @property
-    def recursive_config_types(self):
-        return []
-
     def post_process(self, value):
+        '''
+        Implement this in order to take a value provided by the user
+        and perform computation on it. This can be done to coerce data types,
+        fetch things from the environment (e.g. environment variables), or
+        to do custom validation. If the value is not valid, throw a
+        PostProcessingError. Otherwise return the coerced value.
+        '''
         return value
 
 
+@whitelist_for_serdes
+class ConfigScalarKind(PythonEnum):
+    INT = 'INT'
+    STRING = 'STRING'
+    FLOAT = 'FLOAT'
+    BOOL = 'BOOL'
+
+
 # Scalars, Composites, Selectors, Lists, Optional, Any
 
 
 class ConfigScalar(ConfigType):
-    def __init__(self, key, given_name, **kwargs):
+    def __init__(self, key, given_name, scalar_kind, **kwargs):
+        self.scalar_kind = check.inst_param(scalar_kind, 'scalar_kind', ConfigScalarKind)
         super(ConfigScalar, self).__init__(
             key, given_name=given_name, kind=ConfigTypeKind.SCALAR, **kwargs
         )
 
 
 class BuiltinConfigScalar(ConfigScalar):
-    def __init__(self, description=None):
+    def __init__(self, scalar_kind, description=None):
         super(BuiltinConfigScalar, self).__init__(
-            key=type(self).__name__, given_name=type(self).__name__, description=description,
+            key=type(self).__name__,
+            given_name=type(self).__name__,
+            scalar_kind=scalar_kind,
+            description=description,
         )
 
 
 class Int(BuiltinConfigScalar):
     def __init__(self):
-        super(Int, self).__init__(description='')
+        super(Int, self).__init__(scalar_kind=ConfigScalarKind.INT, description='')
 
 
-class _StringishBuiltin(BuiltinConfigScalar):
-    pass
-
-
-class String(_StringishBuiltin):
+class String(BuiltinConfigScalar):
     def __init__(self):
-        super(String, self).__init__(description='')
-
-
-class Path(_StringishBuiltin):
-    def __init__(self):
-        super(Path, self).__init__(description='')
+        super(String, self).__init__(scalar_kind=ConfigScalarKind.STRING, description='')
 
 
 class Bool(BuiltinConfigScalar):
     def __init__(self):
-        super(Bool, self).__init__(description='')
+        super(Bool, self).__init__(scalar_kind=ConfigScalarKind.BOOL, description='')
 
 
 class Float(BuiltinConfigScalar):
     def __init__(self):
-        super(Float, self).__init__(description='')
+        super(Float, self).__init__(scalar_kind=ConfigScalarKind.FLOAT, description='')
 
     def post_process(self, value):
         return float(value)
@@ -139,10 +149,6 @@ class Noneable(ConfigType):
             type_params=[self.inner_type],
         )
 
-    @property
-    def recursive_config_types(self):
-        return [self.inner_type] + self.inner_type.recursive_config_types
-
 
 class Array(ConfigType):
     def __init__(self, inner_type):
@@ -157,15 +163,7 @@ class Array(ConfigType):
 
     @property
     def description(self):
-        from .type_printer import print_config_type_to_string
-
-        return 'List of {inner_type}'.format(
-            inner_type=print_config_type_to_string(self, with_lines=False)
-        )
-
-    @property
-    def recursive_config_types(self):
-        return [self.inner_type] + self.inner_type.recursive_config_types
+        return 'List of {inner_type}'.format(inner_type=self.key)
 
 
 class EnumValue(object):
@@ -198,7 +196,7 @@ class Enum(ConfigType):
         .. code-block:: python
 
             @solid(
-                config=Field(
+                config_schema=Field(
                     Enum(
                         'CowboyType',
                         [
@@ -263,7 +261,7 @@ class Enum(ConfigType):
                     BLUE = enum.auto()
 
                 @solid(
-                    config={"color": Field(Enum.from_python_enum(Color))}
+                    config_schema={"color": Field(Enum.from_python_enum(Color))}
                 )
                 def select_color(context):
                     # ...
@@ -275,25 +273,29 @@ class Enum(ConfigType):
 
 class ScalarUnion(ConfigType):
     def __init__(
-        self, scalar_type, non_scalar_type, _key=None,
+        self, scalar_type, non_scalar_schema, _key=None,
     ):
-        self.scalar_type = check.inst_param(scalar_type, 'scalar_type', ConfigType)
-        self.non_scalar_type = check.inst_param(non_scalar_type, 'non_scalar_type', ConfigType)
+        from .field import resolve_to_config_type
 
-        check.param_invariant(scalar_type.kind == ConfigTypeKind.SCALAR, 'scalar_type')
+        self.scalar_type = resolve_to_config_type(scalar_type)
+        self.non_scalar_type = resolve_to_config_type(non_scalar_schema)
+
+        check.param_invariant(self.scalar_type.kind == ConfigTypeKind.SCALAR, 'scalar_type')
         check.param_invariant(
-            non_scalar_type.kind
+            self.non_scalar_type.kind
             in {ConfigTypeKind.STRICT_SHAPE, ConfigTypeKind.SELECTOR, ConfigTypeKind.ARRAY},
             'non_scalar_type',
         )
 
         # https://github.com/dagster-io/dagster/issues/2133
         key = check.opt_str_param(
-            _key, '_key', 'ScalarUnion.{}-{}'.format(scalar_type.key, non_scalar_type.key)
+            _key, '_key', 'ScalarUnion.{}-{}'.format(self.scalar_type.key, self.non_scalar_type.key)
         )
 
         super(ScalarUnion, self).__init__(
-            key=key, kind=ConfigTypeKind.SCALAR_UNION, type_params=[scalar_type, non_scalar_type],
+            key=key,
+            kind=ConfigTypeKind.SCALAR_UNION,
+            type_params=[self.scalar_type, self.non_scalar_type],
         )
 
 
@@ -301,15 +303,28 @@ ConfigAnyInstance = Any()
 ConfigBoolInstance = Bool()
 ConfigFloatInstance = Float()
 ConfigIntInstance = Int()
-ConfigPathInstance = Path()
 ConfigStringInstance = String()
 _CONFIG_MAP = {
     BuiltinEnum.ANY: ConfigAnyInstance,
     BuiltinEnum.BOOL: ConfigBoolInstance,
     BuiltinEnum.FLOAT: ConfigFloatInstance,
     BuiltinEnum.INT: ConfigIntInstance,
-    BuiltinEnum.PATH: ConfigPathInstance,
     BuiltinEnum.STRING: ConfigStringInstance,
 }
 
+
+_CONFIG_MAP_BY_NAME = {
+    'Any': ConfigAnyInstance,
+    'Bool': ConfigBoolInstance,
+    'Float': ConfigFloatInstance,
+    'Int': ConfigIntInstance,
+    'String': ConfigStringInstance,
+}
+
 ALL_CONFIG_BUILTINS = set(_CONFIG_MAP.values())
+
+
+def get_builtin_scalar_by_name(type_name):
+    if type_name not in _CONFIG_MAP_BY_NAME:
+        check.failed('Scalar {} is not supported'.format(type_name))
+    return _CONFIG_MAP_BY_NAME[type_name]
