@@ -4,53 +4,60 @@ from collections import defaultdict, namedtuple
 import six
 
 from dagster import check
-from dagster.utils import camelcase
 from dagster.core.errors import DagsterInvalidDefinitionError
+from dagster.serdes import whitelist_for_serdes
+from dagster.utils import frozentags
 
 from .input import InputDefinition
 from .output import OutputDefinition
-from .utils import DEFAULT_OUTPUT, struct_to_string
+from .utils import DEFAULT_OUTPUT, struct_to_string, validate_tags
 
 
-class SolidInvocation(namedtuple('Solid', 'name alias resource_mapper_fn')):
-    '''
-    A solid identifier in a dependency structure. Allows supplying parameters to the solid,
-    like the alias.
+class SolidInvocation(namedtuple('Solid', 'name alias tags')):
+    '''Identifies an instance of a solid in a pipeline dependency structure.
 
     Args:
-        name (str): Name of the solid in the pipeline to instance.
-        alias (Optional[str]):
-            Name for this instance of the solid. Necessary when there are multiple instances
-            of the same solid.
+        name (str): Name of the solid of which this is an instance.
+        alias (Optional[str]): Name specific to this instance of the solid. Necessary when there are
+            multiple instances of the same solid.
+        tags (Optional[Dict[str, Any]]): Optional tags values to extend or override those
+            set on the solid definition.
 
-    Example:
+    Examples:
 
         .. code-block:: python
 
-            pipeline = Pipeline(
+            pipeline = PipelineDefinition(
                 solid_defs=[solid_1, solid_2]
                 dependencies={
-                    SolidInvocation('solid_2', alias='other_name') : {
-                        'input_name' : DependencyDefinition('solid_2'),
+                    SolidInvocation('solid_1', alias='other_name') : {
+                        'input_name' : DependencyDefinition('solid_1'),
                     },
-                    'solid_1' : {
+                    'solid_2' : {
                         'input_name': DependencyDefinition('other_name'),
                     },
                 }
             )
+
+    In general, users should prefer not to construct this class directly or use the
+    :py:class:`PipelineDefinition` API that requires instances of this class. Instead, use the
+    :py:func:`@pipeline <pipeline>` API:
+
+    .. code-block:: python
+
+        @pipeline
+        def pipeline():
+            other_name = solid_1.alias('other_name')
+            solid_2(other_name(solid_1))
+
     '''
 
-    def __new__(cls, name, alias=None, resource_mapper_fn=None):
+    def __new__(cls, name, alias=None, tags=None):
         name = check.str_param(name, 'name')
         alias = check.opt_str_param(alias, 'alias')
-        resource_mapper_fn = check.opt_callable_param(
-            resource_mapper_fn, 'resource_mapper_fn', SolidInvocation.default_resource_mapper_fn
-        )
-        return super(cls, SolidInvocation).__new__(cls, name, alias, resource_mapper_fn)
+        tags = frozentags(check.opt_dict_param(tags, 'tags', value_type=str, key_type=str))
 
-    @staticmethod
-    def default_resource_mapper_fn(resources, resource_deps):
-        return {r: resources.get(r) for r in resource_deps}
+        return super(cls, SolidInvocation).__new__(cls, name, alias, tags)
 
 
 class Solid(object):
@@ -64,15 +71,17 @@ class Solid(object):
             Definition of the solid.
     '''
 
-    def __init__(self, name, definition, resource_mapper_fn, container_definition=None):
+    def __init__(
+        self, name, definition, container_definition=None, tags=None,
+    ):
         from .solid import ISolidDefinition, CompositeSolidDefinition
 
         self.name = check.str_param(name, 'name')
         self.definition = check.inst_param(definition, 'definition', ISolidDefinition)
-        self.resource_mapper_fn = check.callable_param(resource_mapper_fn, 'resource_mapper_fn')
         self.container_definition = check.opt_inst_param(
             container_definition, 'container_definition', CompositeSolidDefinition
         )
+        self._additional_tags = validate_tags(tags)
 
         input_handles = {}
         for name, input_def in self.definition.input_dict.items():
@@ -112,6 +121,12 @@ class Solid(object):
     def output_def_named(self, name):
         return self.definition.output_def_named(name)
 
+    @property
+    def is_composite(self):
+        from .solid import CompositeSolidDefinition
+
+        return isinstance(self.definition, CompositeSolidDefinition)
+
     @property
     def input_dict(self):
         return self.definition.input_dict
@@ -121,8 +136,8 @@ class Solid(object):
         return self.definition.output_dict
 
     @property
-    def step_metadata_fn(self):
-        return self.definition.step_metadata_fn
+    def tags(self):
+        return self.definition.tags.updated_with(self._additional_tags)
 
     def container_maps_input(self, input_name):
         if self.container_definition is None:
@@ -134,29 +149,125 @@ class Solid(object):
         return self.container_definition.mapped_input(self.name, input_name)
 
 
-class SolidHandle(namedtuple('_SolidHandle', 'name definition_name parent')):
-    def __new__(cls, name, definition_name, parent):
+@whitelist_for_serdes
+class SolidHandle(namedtuple('_SolidHandle', 'name parent')):
+    def __new__(cls, name, parent):
         return super(SolidHandle, cls).__new__(
-            cls,
-            check.str_param(name, 'name'),
-            check.opt_str_param(definition_name, 'definition_name'),
-            check.opt_inst_param(parent, 'parent', SolidHandle),
+            cls, check.str_param(name, 'name'), check.opt_inst_param(parent, 'parent', SolidHandle),
         )
 
     def __str__(self):
         return self.to_string()
 
+    @property
+    def path(self):
+        '''Return a list representation of the handle.
+
+        Inverse of SolidHandle.from_path.
+
+        Returns:
+            List[str]:
+        '''
+        path = []
+        cur = self
+        while cur:
+            path.append(cur.name)
+            cur = cur.parent
+        path.reverse()
+        return path
+
     def to_string(self):
-        # Return unique name of the solid and its lineage (omits solid definition names)
+        '''Return a unique string representation of the handle.
+
+        Inverse of SolidHandle.from_string.
+        '''
         return self.parent.to_string() + '.' + self.name if self.parent else self.name
 
-    def camelcase(self):
-        return (
-            self.parent.camelcase() + '.' + camelcase(self.name)
-            if self.parent
-            else camelcase(self.name)
+    def is_or_descends_from(self, handle):
+        '''Check if the handle is or descends from another handle.
+
+        Args:
+            handle (SolidHandle): The handle to check against.
+
+        Returns:
+            bool:
+        '''
+        check.inst_param(handle, 'handle', SolidHandle)
+
+        for idx in range(len(handle.path)):
+            if idx >= len(self.path):
+                return False
+            if self.path[idx] != handle.path[idx]:
+                return False
+        return True
+
+    def pop(self, ancestor):
+        '''Return a copy of the handle with some of its ancestors pruned.
+
+        Args:
+            ancestor (SolidHandle): Handle to an ancestor of the current handle.
+
+        Returns:
+            SolidHandle:
+
+        Example:
+
+        .. code-block:: python
+
+            handle = SolidHandle('baz', SolidHandle('bar', SolidHandle('foo', None)))
+            ancestor = SolidHandle('bar', SolidHandle('foo', None))
+            assert handle.pop(ancestor) == SolidHandle('baz', None)
+        '''
+
+        check.inst_param(ancestor, 'ancestor', SolidHandle)
+        check.invariant(
+            self.is_or_descends_from(ancestor),
+            'Handle {handle} does not descend from {ancestor}'.format(
+                handle=self.to_string(), ancestor=ancestor.to_string()
+            ),
         )
 
+        return SolidHandle.from_path(self.path[len(ancestor.path) :])
+
+    def with_ancestor(self, ancestor):
+        '''Returns a copy of the handle with an ancestor grafted on.
+
+        Args:
+            ancestor (SolidHandle): Handle to the new ancestor.
+
+        Returns:
+            SolidHandle:
+
+        Example:
+
+        .. code-block:: python
+
+            handle = SolidHandle('baz', SolidHandle('bar', SolidHandle('foo', None)))
+            ancestor = SolidHandle('quux' None)
+            assert handle.with_ancestor(ancestor) == SolidHandle(
+                'baz', SolidHandle('bar', SolidHandle('foo', SolidHandle('quux', None)))
+            )
+        '''
+        check.opt_inst_param(ancestor, 'ancestor', SolidHandle)
+
+        return SolidHandle.from_path((ancestor.path if ancestor else []) + self.path)
+
+    @staticmethod
+    def from_path(path):
+        check.list_param(path, 'path', of_type=str)
+
+        cur = None
+        while len(path) > 0:
+            cur = SolidHandle(name=path.pop(0), parent=cur)
+        return cur
+
+    @staticmethod
+    def from_string(handle_str):
+        check.str_param(handle_str, 'handle_str')
+
+        path = handle_str.split('.')
+        return SolidHandle.from_path(path)
+
     @classmethod
     def from_dict(cls, dict_repr):
         '''This method makes it possible to rehydrate a potentially nested SolidHandle after a
@@ -166,24 +277,16 @@ class SolidHandle(namedtuple('_SolidHandle', 'name definition_name parent')):
         check.invariant(
             'name' in dict_repr, 'Dict representation of SolidHandle must have a \'name\' key'
         )
-        check.invariant(
-            'definition_name' in dict_repr,
-            'Dict representation of SolidHandle must have a \'definition_name\' key',
-        )
         check.invariant(
             'parent' in dict_repr, 'Dict representation of SolidHandle must have a \'parent\' key'
         )
 
         if isinstance(dict_repr['parent'], (list, tuple)):
             dict_repr['parent'] = SolidHandle.from_dict(
-                {
-                    'name': dict_repr['parent'][0],
-                    'definition_name': dict_repr['parent'][1],
-                    'parent': dict_repr['parent'][2],
-                }
+                {'name': dict_repr['parent'][0], 'parent': dict_repr['parent'][1],}
             )
 
-        return SolidHandle(**dict_repr)
+        return SolidHandle(**{k: dict_repr[k] for k in ['name', 'parent']})
 
 
 class SolidInputHandle(namedtuple('_SolidInputHandle', 'solid input_def')):
@@ -196,10 +299,7 @@ class SolidInputHandle(namedtuple('_SolidInputHandle', 'solid input_def')):
 
     def _inner_str(self):
         return struct_to_string(
-            'SolidInputHandle',
-            solid_name=self.solid.name,
-            definition_name=self.solid.definition.name,
-            input_name=self.input_def.name,
+            'SolidInputHandle', solid_name=self.solid.name, input_name=self.input_def.name,
         )
 
     def __str__(self):
@@ -214,6 +314,10 @@ class SolidInputHandle(namedtuple('_SolidInputHandle', 'solid input_def')):
     def __eq__(self, other):
         return self.solid.name == other.solid.name and self.input_def.name == other.input_def.name
 
+    @property
+    def solid_name(self):
+        return self.solid.name
+
 
 class SolidOutputHandle(namedtuple('_SolidOutputHandle', 'solid output_def')):
     def __new__(cls, solid, output_def):
@@ -225,10 +329,7 @@ class SolidOutputHandle(namedtuple('_SolidOutputHandle', 'solid output_def')):
 
     def _inner_str(self):
         return struct_to_string(
-            'SolidOutputHandle',
-            solid_name=self.solid.name,
-            definition_name=self.solid.definition.name,
-            output_name=self.output_def.name,
+            'SolidOutputHandle', solid_name=self.solid.name, output_name=self.output_def.name,
         )
 
     def __str__(self):
@@ -243,6 +344,10 @@ class SolidOutputHandle(namedtuple('_SolidOutputHandle', 'solid output_def')):
     def __eq__(self, other):
         return self.solid.name == other.solid.name and self.output_def.name == other.output_def.name
 
+    @property
+    def solid_name(self):
+        return self.solid.name
+
 
 class InputToOutputHandleDict(defaultdict):
     def __init__(self):
@@ -289,45 +394,62 @@ def _create_handle_dict(solid_dict, dep_dict):
 class DependencyStructure(object):
     @staticmethod
     def from_definitions(solids, dep_dict):
-        return DependencyStructure(_create_handle_dict(solids, dep_dict))
+        return DependencyStructure(list(dep_dict.keys()), _create_handle_dict(solids, dep_dict))
 
-    def __init__(self, handle_dict):
+    def __init__(self, solid_names, handle_dict):
+        self._solid_names = solid_names
         self._handle_dict = check.inst_param(handle_dict, 'handle_dict', InputToOutputHandleDict)
 
-    def deps_of_solid(self, solid_name):
-        check.str_param(solid_name, 'solid_name')
+        # Building up a couple indexes here so that one can look up all the upstream output handles
+        # or downstream input handles in O(1). Without this, this can become O(N^2) where N is solid
+        # count during the GraphQL query in particular
 
-        return list(handles[1] for handles in self.__gen_deps_of_solid(solid_name))
+        # solid_name => input_handle => list[output_handle]
+        self._solid_input_index = defaultdict(dict)
 
-    def deps_of_solid_with_input(self, solid_name):
-        check.str_param(solid_name, 'solid_name')
-        result = defaultdict(list)
-        for input_handle, output_handle in self.__gen_deps_of_solid(solid_name):
-            result[input_handle].append(output_handle)
-        return result
+        # solid_name => output_handle => list[input_handle]
+        self._solid_output_index = defaultdict(lambda: defaultdict(list))
 
-    def __gen_deps_of_solid(self, solid_name):
         for input_handle, output_handle_or_list in self._handle_dict.items():
-            if input_handle.solid.name == solid_name:
-                if isinstance(output_handle_or_list, list):
-                    for output_handle in output_handle_or_list:
-                        yield (input_handle, output_handle)
-                else:
-                    yield (input_handle, output_handle_or_list)
+            output_handle_list = (
+                output_handle_or_list
+                if isinstance(output_handle_or_list, list)
+                else [output_handle_or_list]
+            )
+            self._solid_input_index[input_handle.solid.name][input_handle] = output_handle_list
+            for output_handle in output_handle_list:
+                self._solid_output_index[output_handle.solid.name][output_handle].append(
+                    input_handle
+                )
 
-    def depended_by_of_solid(self, solid_name):
+    def all_upstream_outputs_from_solid(self, solid_name):
         check.str_param(solid_name, 'solid_name')
-        result = defaultdict(list)
-        for input_handle, output_handle_or_list in self._handle_dict.items():
-            if isinstance(output_handle_or_list, list):
-                for output_handle in output_handle_or_list:
-                    if output_handle.solid.name == solid_name:
-                        result[output_handle].append(input_handle)
-            else:
-                if output_handle_or_list.solid.name == solid_name:
-                    result[output_handle_or_list].append(input_handle)
 
-        return result
+        # flatten out all outputs that feed into the inputs of this solid
+        return [
+            output_handle
+            for output_handle_list in self._solid_input_index[solid_name].values()
+            for output_handle in output_handle_list
+        ]
+
+    def input_to_upstream_outputs_for_solid(self, solid_name):
+        '''
+        Returns a Dict[SolidInputHandle, List[SolidOutputHandle]] that encodes
+        where all the the inputs are sourced from upstream. Usually the
+        List[SolidOutputHandle] will be a list of one, except for the
+        multi-dependency case.
+        '''
+        check.str_param(solid_name, 'solid_name')
+        return self._solid_input_index[solid_name]
+
+    def output_to_downstream_inputs_for_solid(self, solid_name):
+        '''
+        Returns a Dict[SolidOutputHandle, List[SolidInputHandle]] that
+        represents all the downstream inputs for each output in the
+        dictionary
+        '''
+        check.str_param(solid_name, 'solid_name')
+        return self._solid_output_index[solid_name]
 
     def has_singular_dep(self, solid_input_handle):
         check.inst_param(solid_input_handle, 'solid_input_handle', SolidInputHandle)
@@ -375,6 +497,20 @@ class DependencyStructure(object):
     def items(self):
         return self._handle_dict.items()
 
+    def debug_str(self):
+        if not self.items():
+            return 'DependencyStructure: EMPTY'
+
+        debug = 'DependencyStructure: \n'
+        for in_handle, out_handle in self.items():
+            debug += '  {out_solid}.{out_name} ---> {in_solid}.{in_name}\n'.format(
+                out_solid=out_handle.solid.name,
+                out_name=out_handle.output_def.name,
+                in_name=in_handle.input_def.name,
+                in_solid=in_handle.solid.name,
+            )
+        return debug
+
 
 class IDependencyDefinition(six.with_metaclass(ABCMeta)):  # pylint: disable=no-init
     @abstractmethod
@@ -389,20 +525,39 @@ class IDependencyDefinition(six.with_metaclass(ABCMeta)):  # pylint: disable=no-
 class DependencyDefinition(
     namedtuple('_DependencyDefinition', 'solid output description'), IDependencyDefinition
 ):
-    '''Dependency definitions represent an edge in the DAG of solids. This object is
-    used with a dictionary structure (whose keys represent solid/input where the dependency
-    comes from) so this object only contains the target dependency information.
+    '''Represents an edge in the DAG of solid instances forming a pipeline.
+
+    This object is used at the leaves of a dictionary structure that represents the complete
+    dependency structure of a pipeline whose keys represent the dependent solid and dependent
+    input, so this object only contains information about the dependee.
+
+    Concretely, if the input named 'input' of solid_b depends on the output named 'result' of
+    solid_a, this structure will look as follows:
+
+    .. code-block:: python
+
+        dependency_structure = {
+            'solid_b': {
+                'input': DependencyDefinition('solid_a', 'result')
+            }
+        }
+
+    In general, users should prefer not to construct this class directly or use the
+    :py:class:`PipelineDefinition` API that requires instances of this class. Instead, use the
+    :py:func:`@pipeline <pipeline>` API:
+
+    .. code-block:: python
+
+        @pipeline
+        def pipeline():
+            solid_b(solid_a())
+
 
     Args:
-        solid (str):
-            The name of the solid that is the target of the dependency.
-            This is the solid where the value passed between the solids
-            comes from.
-        output (str):
-            The name of the output that is the target of the dependency.
-            Defaults to "result", the default output name of solids with a single output.
-        description (str):
-            Description of this dependency. Optional.
+        solid (str): The name of the solid that is depended on, that is, from which the value
+            passed between the two solids originates.
+        output (Optional[str]): The name of the output that is depended on. (default: "result")
+        description (Optional[str]): Human-readable description of this dependency.
     '''
 
     def __new__(cls, solid, output=DEFAULT_OUTPUT, description=None):
@@ -423,6 +578,48 @@ class DependencyDefinition(
 class MultiDependencyDefinition(
     namedtuple('_MultiDependencyDefinition', 'dependencies'), IDependencyDefinition
 ):
+    '''Represents a fan-in edge in the DAG of solid instances forming a pipeline.
+
+    This object is used only when an input of type ``List[T]`` is assembled by fanning-in multiple
+    upstream outputs of type ``T``.
+
+    This object is used at the leaves of a dictionary structure that represents the complete
+    dependency structure of a pipeline whose keys represent the dependent solid and dependent
+    input, so this object only contains information about the dependee.
+
+    Concretely, if the input named 'input' of solid_c depends on the outputs named 'result' of
+    solid_a and solid_b, this structure will look as follows:
+
+    .. code-block:: python
+
+        dependency_structure = {
+            'solid_c': {
+                'input': MultiDependencyDefinition(
+                    [
+                        DependencyDefinition('solid_a', 'result'),
+                        DependencyDefinition('solid_b', 'result')
+                    ]
+                )
+            }
+        }
+
+    In general, users should prefer not to construct this class directly or use the
+    :py:class:`PipelineDefinition` API that requires instances of this class. Instead, use the
+    :py:func:`@pipeline <pipeline>` API:
+
+    .. code-block:: python
+
+        @pipeline
+        def pipeline():
+            solid_c(solid_a(), solid_b())
+
+    Args:
+        solid (str): The name of the solid that is depended on, that is, from which the value
+            passed between the two solids originates.
+        output (Optional[str]): The name of the output that is depended on. (default: "result")
+        description (Optional[str]): Human-readable description of this dependency.
+    '''
+
     def __new__(cls, dependencies):
         deps = check.list_param(dependencies, 'dependencies', of_type=DependencyDefinition)
         seen = {}
