@@ -4,50 +4,41 @@ from dagster import check
 from dagster.builtins import Int
 from dagster.config.field import Field
 from dagster.config.field_utils import check_user_facing_opt_config_param
+from dagster.core.definitions.reconstructable import ReconstructablePipeline
 from dagster.core.errors import DagsterUnmetExecutorRequirementsError
-from dagster.core.execution.config import InProcessExecutorConfig, MultiprocessExecutorConfig
 from dagster.core.execution.retries import Retries, get_retries_config
+from dagster.utils.backcompat import canonicalize_backcompat_args, rename_warning
 
 
 class ExecutorDefinition(object):
     '''
     Args:
         name (Optional[str]): The name of the executor.
-        config (Optional[Any]): The schema for the config. Configuration data available in
-            `init_context.executor_config`.
-
-            This value can be any of:
-
-            1. A Python primitive type that resolves to a Dagster config type
-               (:py:class:`~python:int`, :py:class:`~python:float`, :py:class:`~python:bool`,
-               :py:class:`~python:str`, or :py:class:`~python:list`).
-
-            2. A Dagster config type: :py:data:`~dagster.Int`, :py:data:`~dagster.Float`,
-               :py:data:`~dagster.Bool`, :py:data:`~dagster.String`,
-               :py:data:`~dagster.StringSource`, :py:data:`~dagster.Path`, :py:data:`~dagster.Any`,
-               :py:class:`~dagster.Array`, :py:data:`~dagster.Noneable`, :py:data:`~dagster.Enum`,
-               :py:class:`~dagster.Selector`, :py:class:`~dagster.Shape`, or
-               :py:class:`~dagster.Permissive`.
-
-            3. A bare python dictionary, which will be automatically wrapped in
-               :py:class:`~dagster.Shape`. Values of the dictionary are resolved recursively
-               according to the same rules.
-
-            4. A bare python list of length one which itself is config type.
-               Becomes :py:class:`Array` with list element as an argument.
-
-            5. An instance of :py:class:`~dagster.Field`.
-
+        config_schema (Optional[ConfigSchema]): The schema for the config. Configuration data
+            available in `init_context.executor_config`.
         executor_creation_fn(Optional[Callable]): Should accept an :py:class:`InitExecutorContext`
-            and return an instance of :py:class:`ExecutorConfig`.
+            and return an instance of :py:class:`Executor`
         required_resource_keys (Optional[Set[str]]): Keys for the resources required by the
             executor.
 
     '''
 
-    def __init__(self, name, config=None, executor_creation_fn=None, required_resource_keys=None):
+    def __init__(
+        self,
+        name,
+        config_schema=None,
+        executor_creation_fn=None,
+        required_resource_keys=None,
+        config=None,
+    ):
         self._name = check.str_param(name, 'name')
-        self._config_field = check_user_facing_opt_config_param(config, 'config')
+        self._config_schema = canonicalize_backcompat_args(
+            check_user_facing_opt_config_param(config_schema, 'config_schema'),
+            'config_schema',
+            check_user_facing_opt_config_param(config, 'config'),
+            'config',
+            '0.9.0',
+        )
         self._executor_creation_fn = check.opt_callable_param(
             executor_creation_fn, 'executor_creation_fn'
         )
@@ -61,7 +52,12 @@ class ExecutorDefinition(object):
 
     @property
     def config_field(self):
-        return self._config_field
+        rename_warning('config_schema', 'config_field', '0.9.0')
+        return self._config_schema
+
+    @property
+    def config_schema(self):
+        return self._config_schema
 
     @property
     def executor_creation_fn(self):
@@ -72,56 +68,38 @@ class ExecutorDefinition(object):
         return self._required_resource_keys
 
 
-def executor(name=None, config=None, required_resource_keys=None):
+def executor(name=None, config_schema=None, required_resource_keys=None, config=None):
     '''Define an executor.
 
     The decorated function should accept an :py:class:`InitExecutorContext` and return an instance
-    of :py:class:`ExecutorConfig`.
+    of :py:class:`Executor`.
 
     Args:
         name (Optional[str]): The name of the executor.
-        config (Optional[Any]): The schema for the config. Configuration data available in
+        config_schema (Optional[ConfigSchema]): The schema for the config. Configuration data available in
             `init_context.executor_config`.
-
-            This value can be any of:
-
-            1. A Python primitive type that resolves to a Dagster config type
-               (:py:class:`~python:int`, :py:class:`~python:float`, :py:class:`~python:bool`,
-               :py:class:`~python:str`, or :py:class:`~python:list`).
-
-            2. A Dagster config type: :py:data:`~dagster.Int`, :py:data:`~dagster.Float`,
-               :py:data:`~dagster.Bool`, :py:data:`~dagster.String`,
-               :py:data:`~dagster.StringSource`, :py:data:`~dagster.Path`, :py:data:`~dagster.Any`,
-               :py:class:`~dagster.Array`, :py:data:`~dagster.Noneable`, :py:data:`~dagster.Enum`,
-               :py:class:`~dagster.Selector`, :py:class:`~dagster.Shape`, or
-               :py:class:`~dagster.Permissive`.
-
-            3. A bare python dictionary, which will be automatically wrapped in
-               :py:class:`~dagster.Shape`. Values of the dictionary are resolved recursively
-               according to the same rules.
-
-            4. A bare python list of length one which itself is config type.
-               Becomes :py:class:`Array` with list element as an argument.
-
-            5. An instance of :py:class:`~dagster.Field`.
-
         required_resource_keys (Optional[Set[str]]): Keys for the resources required by the
             executor.
     '''
     if callable(name):
-        check.invariant(config is None)
+        check.invariant(config_schema is None)
         check.invariant(required_resource_keys is None)
+        check.invariant(config is None)
         return _ExecutorDecoratorCallable()(name)
 
     return _ExecutorDecoratorCallable(
-        name=name, config=config, required_resource_keys=required_resource_keys,
+        name=name,
+        config_schema=canonicalize_backcompat_args(
+            config_schema, 'config_schema', config, 'config', '0.9.0'
+        ),
+        required_resource_keys=required_resource_keys,
     )
 
 
 class _ExecutorDecoratorCallable(object):
-    def __init__(self, name=None, config=None, required_resource_keys=None):
+    def __init__(self, name=None, config_schema=None, required_resource_keys=None):
         self.name = check.opt_str_param(name, 'name')
-        self.config = config  # type check in definition
+        self.config_schema = config_schema  # type check in definition
         self.required_resource_keys = required_resource_keys  # type check in definition
 
     def __call__(self, fn):
@@ -132,7 +110,7 @@ class _ExecutorDecoratorCallable(object):
 
         executor_def = ExecutorDefinition(
             name=self.name,
-            config=self.config,
+            config_schema=self.config_schema,
             executor_creation_fn=fn,
             required_resource_keys=self.required_resource_keys,
         )
@@ -144,7 +122,10 @@ class _ExecutorDecoratorCallable(object):
 
 @executor(
     name='in_process',
-    config={'retries': get_retries_config(), 'marker_to_close': Field(str, is_required=False),},
+    config_schema={
+        'retries': get_retries_config(),
+        'marker_to_close': Field(str, is_required=False),
+    },
 )
 def in_process_executor(init_context):
     '''The default in-process executor.
@@ -162,11 +143,12 @@ def in_process_executor(init_context):
     where the higher the number the higher the priority. 0 is the default and both positive
     and negative numbers can be used.
     '''
-    from dagster.core.engine.init import InitExecutorContext
+    from dagster.core.executor.init import InitExecutorContext
+    from dagster.core.executor.in_process import InProcessExecutor
 
     check.inst_param(init_context, 'init_context', InitExecutorContext)
 
-    return InProcessExecutorConfig(
+    return InProcessExecutor(
         # shouldn't need to .get() here - issue with defaults in config setup
         retries=Retries.from_config(init_context.executor_config.get('retries', {'enabled': {}})),
         marker_to_close=init_context.executor_config.get('marker_to_close'),
@@ -175,7 +157,7 @@ def in_process_executor(init_context):
 
 @executor(
     name='multiprocess',
-    config={
+    config_schema={
         'max_concurrent': Field(Int, is_required=False, default_value=0),
         'retries': get_retries_config(),
     },
@@ -201,21 +183,15 @@ def multiprocess_executor(init_context):
     where the higher the number the higher the priority. 0 is the default and both positive
     and negative numbers can be used.
     '''
-    from dagster.core.definitions.handle import ExecutionTargetHandle
-    from dagster.core.engine.init import InitExecutorContext
+    from dagster.core.executor.init import InitExecutorContext
+    from dagster.core.executor.multiprocess import MultiprocessExecutor
 
     check.inst_param(init_context, 'init_context', InitExecutorContext)
 
     check_cross_process_constraints(init_context)
 
-    # ExecutionTargetHandle.get_handle returns an ExecutionTargetHandleCacheEntry, which is a tuple
-    # (handle, solid_subset). Right now we are throwing away the solid_subset that we store in the
-    # cache -- this is fragile and we should fix this with
-    # https://github.com/dagster-io/dagster/issues/2115 and friends so there are not multiple
-    # sources of truth for the solid subset
-    handle, _ = ExecutionTargetHandle.get_handle(init_context.pipeline_def)
-    return MultiprocessExecutorConfig(
-        handle=handle,
+    return MultiprocessExecutor(
+        pipeline=init_context.pipeline,
         max_concurrent=init_context.executor_config['max_concurrent'],
         retries=Retries.from_config(init_context.executor_config['retries']),
     )
@@ -225,26 +201,25 @@ default_executors = [in_process_executor, multiprocess_executor]
 
 
 def check_cross_process_constraints(init_context):
-    from dagster.core.engine.init import InitExecutorContext
+    from dagster.core.executor.init import InitExecutorContext
 
     check.inst_param(init_context, 'init_context', InitExecutorContext)
 
-    _check_pipeline_has_target_handle(init_context.pipeline_def)
+    _check_intra_process_pipeline(init_context.pipeline)
     _check_non_ephemeral_instance(init_context.instance)
     _check_persistent_storage_requirement(init_context.system_storage_def)
 
 
-def _check_pipeline_has_target_handle(pipeline_def):
-    from dagster.core.definitions.handle import ExecutionTargetHandle
-
-    handle, _ = ExecutionTargetHandle.get_handle(pipeline_def)
-    if not handle:
+def _check_intra_process_pipeline(pipeline):
+    if not isinstance(pipeline, ReconstructablePipeline):
         raise DagsterUnmetExecutorRequirementsError(
             'You have attempted to use an executor that uses multiple processes with the pipeline "{name}" '
-            'that can not be re-hydrated. Pipelines must be loaded in a way that allows dagster to reconstruct '
+            'that is not reconstructable. Pipelines must be loaded in a way that allows dagster to reconstruct '
             'them in a new process. This means: \n'
             '  * using the file, module, or repository.yaml arguments of dagit/dagster-graphql/dagster\n'
-            '  * constructing an ExecutionTargetHandle directly\n'.format(name=pipeline_def.name)
+            '  * loading the pipeline through the reconstructable() function\n'.format(
+                name=pipeline.get_definition().name
+            )
         )
 
 
