@@ -1,15 +1,16 @@
 from collections import namedtuple
 from enum import Enum
 
-
 from dagster import check
-from dagster.core.definitions import SolidHandle, Materialization
-from dagster.core.types.runtime import RuntimeType
-from dagster.utils import merge_dicts
-from dagster.utils.error import SerializableErrorInfo
+from dagster.core.definitions import Materialization, SolidHandle
 from dagster.core.definitions.events import EventMetadataEntry
+from dagster.core.types.dagster_type import DagsterType
+from dagster.serdes import whitelist_for_serdes
+from dagster.utils import frozentags, merge_dicts
+from dagster.utils.error import SerializableErrorInfo
 
 
+@whitelist_for_serdes
 class StepOutputHandle(namedtuple('_StepOutputHandle', 'step_key output_name')):
     @staticmethod
     def from_step(step, output_name='result'):
@@ -25,28 +26,17 @@ class StepOutputHandle(namedtuple('_StepOutputHandle', 'step_key output_name')):
         )
 
 
-class SingleOutputStepCreationData(namedtuple('SingleOutputStepCreationData', 'step output_name')):
-    '''
-    It is very common for step creation to involve processing a single value (e.g. an input thunk).
-    This tuple is meant to be used by those functions to return both a new step and the output
-    that deals with the value in question.
-    '''
-
-    @property
-    def step_output_handle(self):
-        return StepOutputHandle.from_step(self.step, self.output_name)
-
-
-class StepInputData(namedtuple('_StepInputData', 'input_name value_repr type_check_data')):
-    def __new__(cls, input_name, value_repr, type_check_data):
+@whitelist_for_serdes
+class StepInputData(namedtuple('_StepInputData', 'input_name type_check_data')):
+    def __new__(cls, input_name, type_check_data):
         return super(StepInputData, cls).__new__(
             cls,
             input_name=check.str_param(input_name, 'input_name'),
-            value_repr=check.str_param(value_repr, 'value_repr'),
             type_check_data=check.opt_inst_param(type_check_data, 'type_check_data', TypeCheckData),
         )
 
 
+@whitelist_for_serdes
 class TypeCheckData(namedtuple('_TypeCheckData', 'success label description metadata_entries')):
     def __new__(cls, success, label, description=None, metadata_entries=None):
         return super(TypeCheckData, cls).__new__(
@@ -60,6 +50,7 @@ class TypeCheckData(namedtuple('_TypeCheckData', 'success label description meta
         )
 
 
+@whitelist_for_serdes
 class UserFailureData(namedtuple('_UserFailureData', 'label description metadata_entries')):
     def __new__(cls, label, description=None, metadata_entries=None):
         return super(UserFailureData, cls).__new__(
@@ -72,21 +63,16 @@ class UserFailureData(namedtuple('_UserFailureData', 'label description metadata
         )
 
 
+@whitelist_for_serdes
 class StepOutputData(
-    namedtuple(
-        '_StepOutputData',
-        'step_output_handle value_repr intermediate_materialization type_check_data',
-    )
+    namedtuple('_StepOutputData', 'step_output_handle intermediate_materialization type_check_data')
 ):
-    def __new__(
-        cls, step_output_handle, value_repr, intermediate_materialization=None, type_check_data=None
-    ):
+    def __new__(cls, step_output_handle, intermediate_materialization=None, type_check_data=None):
         return super(StepOutputData, cls).__new__(
             cls,
             step_output_handle=check.inst_param(
                 step_output_handle, 'step_output_handle', StepOutputHandle
             ),
-            value_repr=check.str_param(value_repr, 'value_repr'),
             intermediate_materialization=check.opt_inst_param(
                 intermediate_materialization, 'intermediate_materialization', Materialization
             ),
@@ -98,6 +84,7 @@ class StepOutputData(
         return self.step_output_handle.output_name
 
 
+@whitelist_for_serdes
 class StepFailureData(namedtuple('_StepFailureData', 'error user_failure_data')):
     def __new__(cls, error, user_failure_data):
         return super(StepFailureData, cls).__new__(
@@ -109,6 +96,17 @@ class StepFailureData(namedtuple('_StepFailureData', 'error user_failure_data'))
         )
 
 
+@whitelist_for_serdes
+class StepRetryData(namedtuple('_StepRetryData', 'error seconds_to_wait')):
+    def __new__(cls, error, seconds_to_wait=None):
+        return super(StepRetryData, cls).__new__(
+            cls,
+            error=check.opt_inst_param(error, 'error', SerializableErrorInfo),
+            seconds_to_wait=check.opt_int_param(seconds_to_wait, 'seconds_to_wait'),
+        )
+
+
+@whitelist_for_serdes
 class StepSuccessData(namedtuple('_StepSuccessData', 'duration_ms')):
     def __new__(cls, duration_ms):
         return super(StepSuccessData, cls).__new__(
@@ -116,40 +114,73 @@ class StepSuccessData(namedtuple('_StepSuccessData', 'duration_ms')):
         )
 
 
+@whitelist_for_serdes
 class StepKind(Enum):
     COMPUTE = 'COMPUTE'
-    INPUT_EXPECTATION = 'INPUT_EXPECTATION'
-    OUTPUT_EXPECTATION = 'OUTPUT_EXPECTATION'
-    JOIN = 'JOIN'
-    SERIALIZE = 'SERIALIZE'
-    UNMARSHAL_INPUT = 'UNMARSHAL_INPUT'
-    MARSHAL_OUTPUT = 'MARSHAL_OUTPUT'
 
 
-class StepInput(namedtuple('_StepInput', 'name runtime_type prev_output_handle config_data')):
-    def __new__(cls, name, runtime_type, prev_output_handle=None, config_data=None):
+class StepInputSourceType(Enum):
+    SINGLE_OUTPUT = 'SINGLE_OUTPUT'
+    MULTIPLE_OUTPUTS = 'MULTIPLE_OUTPUTS'
+    CONFIG = 'CONFIG'
+    DEFAULT_VALUE = 'DEFAULT_VALUE'
+
+
+class StepInput(
+    namedtuple('_StepInput', 'name dagster_type source_type source_handles config_data')
+):
+    def __new__(
+        cls, name, dagster_type=None, source_type=None, source_handles=None, config_data=None,
+    ):
         return super(StepInput, cls).__new__(
             cls,
             name=check.str_param(name, 'name'),
-            runtime_type=check.inst_param(runtime_type, 'runtime_type', RuntimeType),
-            prev_output_handle=check.opt_inst_param(
-                prev_output_handle, 'prev_output_handle', StepOutputHandle
+            dagster_type=check.inst_param(dagster_type, 'dagster_type', DagsterType),
+            source_type=check.inst_param(source_type, 'source_type', StepInputSourceType),
+            source_handles=check.opt_list_param(
+                source_handles, 'source_handles', of_type=StepOutputHandle
             ),
             config_data=config_data,  # can be any type
         )
 
     @property
     def is_from_output(self):
-        return bool(self.prev_output_handle)
+        return (
+            self.source_type == StepInputSourceType.SINGLE_OUTPUT
+            or self.source_type == StepInputSourceType.MULTIPLE_OUTPUTS
+        )
+
+    @property
+    def is_from_single_output(self):
+        return self.source_type == StepInputSourceType.SINGLE_OUTPUT
+
+    @property
+    def is_from_config(self):
+        return self.source_type == StepInputSourceType.CONFIG
+
+    @property
+    def is_from_default_value(self):
+        return self.source_type == StepInputSourceType.DEFAULT_VALUE
+
+    @property
+    def is_from_multiple_outputs(self):
+        return self.source_type == StepInputSourceType.MULTIPLE_OUTPUTS
+
+    @property
+    def dependency_keys(self):
+        return {handle.step_key for handle in self.source_handles}
 
 
-class StepOutput(namedtuple('_StepOutput', 'name runtime_type optional')):
-    def __new__(cls, name, runtime_type, optional):
+class StepOutput(namedtuple('_StepOutput', 'name dagster_type optional should_materialize')):
+    def __new__(
+        cls, name, dagster_type=None, optional=None, should_materialize=None,
+    ):
         return super(StepOutput, cls).__new__(
             cls,
             name=check.str_param(name, 'name'),
-            runtime_type=check.inst_param(runtime_type, 'runtime_type', RuntimeType),
             optional=check.bool_param(optional, 'optional'),
+            should_materialize=check.bool_param(should_materialize, 'should_materialize'),
+            dagster_type=check.inst_param(dagster_type, 'dagster_type', DagsterType),
         )
 
 
@@ -158,7 +189,7 @@ class ExecutionStep(
         '_ExecutionStep',
         (
             'pipeline_name key_suffix step_inputs step_input_dict step_outputs step_output_dict '
-            'compute_fn kind solid_handle logging_tags metadata'
+            'compute_fn kind solid_handle logging_tags tags'
         ),
     )
 ):
@@ -171,8 +202,9 @@ class ExecutionStep(
         compute_fn,
         kind,
         solid_handle,
+        solid,
         logging_tags=None,
-        metadata=None,
+        tags=None,
     ):
         return super(ExecutionStep, cls).__new__(
             cls,
@@ -190,11 +222,11 @@ class ExecutionStep(
                     'step_key': str(solid_handle) + '.' + key_suffix,
                     'pipeline': pipeline_name,
                     'solid': solid_handle.name,
-                    'solid_definition': solid_handle.definition_name,
+                    'solid_definition': solid.definition.name,
                 },
                 check.opt_dict_param(logging_tags, 'logging_tags'),
             ),
-            metadata=check.opt_dict_param(metadata, 'metadata', key_type=str, value_type=str),
+            tags=check.opt_inst_param(tags, 'tags', frozentags),
         )
 
     @property
@@ -205,10 +237,6 @@ class ExecutionStep(
     def solid_name(self):
         return self.solid_handle.name
 
-    @property
-    def solid_definition_name(self):
-        return self.solid_handle.definition_name
-
     def has_step_output(self, name):
         check.str_param(name, 'name')
         return name in self.step_output_dict
@@ -224,28 +252,3 @@ class ExecutionStep(
     def step_input_named(self, name):
         check.str_param(name, 'name')
         return self.step_input_dict[name]
-
-
-class ExecutionValueSubplan(
-    namedtuple('ExecutionValueSubplan', 'steps terminal_step_output_handle')
-):
-    '''
-    A frequent pattern in the execution engine is to take a single value (e.g. an input or an output
-    of a compute function) and then flow that value value through a sequence of system-injected
-    steps (e.g. expectations or materializations). This object captures that pattern. It contains
-    all of the steps that comprise that Subplan and also a single output handle that points to
-    output that further steps down the plan can depend on.
-    '''
-
-    def __new__(cls, steps, terminal_step_output_handle):
-        return super(ExecutionValueSubplan, cls).__new__(
-            cls,
-            check.list_param(steps, 'steps', of_type=ExecutionStep),
-            check.inst_param(
-                terminal_step_output_handle, 'terminal_step_output_handle', StepOutputHandle
-            ),
-        )
-
-    @staticmethod
-    def empty(terminal_step_output_handle):
-        return ExecutionValueSubplan([], terminal_step_output_handle)
