@@ -1,11 +1,18 @@
 import pytest
 
-from dagster import DagsterInvalidConfigDefinitionError, Noneable, Selector, execute_solid, solid
+from dagster import (
+    DagsterInvalidConfigDefinitionError,
+    Noneable,
+    Permissive,
+    Selector,
+    execute_solid,
+    solid,
+)
 
 
 def test_kitchen_sink():
     @solid(
-        config={
+        config_schema={
             'str_field': str,
             'int_field': int,
             'list_int': [int],
@@ -36,8 +43,7 @@ def test_kitchen_sink():
 
     assert (
         execute_solid(
-            kitchen_sink,
-            environment_dict={'solids': {'kitchen_sink': {'config': solid_config_one}}},
+            kitchen_sink, run_config={'solids': {'kitchen_sink': {'config': solid_config_one}}},
         ).output_value()
         == solid_config_one
     )
@@ -55,17 +61,33 @@ def test_kitchen_sink():
 
     assert (
         execute_solid(
-            kitchen_sink,
-            environment_dict={'solids': {'kitchen_sink': {'config': solid_config_two}}},
+            kitchen_sink, run_config={'solids': {'kitchen_sink': {'config': solid_config_two}}},
         ).output_value()
         == solid_config_two
     )
 
 
+def test_builtin_dict():
+    executed = {}
+
+    @solid(config_schema=dict)
+    def builtin_dict_solid(context):
+        executed['yup'] = True
+        return context.solid_config
+
+    assert isinstance(builtin_dict_solid.config_field.config_type, Permissive)
+
+    assert execute_solid(
+        builtin_dict_solid, run_config={'solids': {'builtin_dict_solid': {'config': {'a': 'b'}}}}
+    ).output_value() == {'a': 'b'}
+
+    assert executed['yup']
+
+
 def test_bad_solid_config_argument():
     with pytest.raises(DagsterInvalidConfigDefinitionError) as exc_info:
 
-        @solid(config='dkjfkd')
+        @solid(config_schema='dkjfkd')
         def _bad_config(_):
             pass
 
@@ -77,7 +99,7 @@ def test_bad_solid_config_argument():
 def test_bad_solid_config_argument_nested():
     with pytest.raises(DagsterInvalidConfigDefinitionError) as exc_info:
 
-        @solid(config={'field': 'kdjkfjd'})
+        @solid(config_schema={'field': 'kdjkfjd'})
         def _bad_config(_):
             pass
 
@@ -90,7 +112,7 @@ def test_bad_solid_config_argument_nested():
 def test_bad_solid_config_argument_list_wrong_length():
     with pytest.raises(DagsterInvalidConfigDefinitionError) as exc_info:
 
-        @solid(config={'bad_list': []})
+        @solid(config_schema={'bad_list': []})
         def _bad_list_config(_):
             pass
 
@@ -105,7 +127,7 @@ def test_bad_solid_config_argument_list_bad_item():
 
     with pytest.raises(DagsterInvalidConfigDefinitionError) as exc_info:
 
-        @solid(config={'bad_list': ['kdjfkd']})
+        @solid(config_schema={'bad_list': ['kdjfkd']})
         def _bad_list_config(_):
             pass
 
@@ -121,7 +143,7 @@ def test_bad_solid_config_argument_list_bad_nested_item():
 
     with pytest.raises(DagsterInvalidConfigDefinitionError) as exc_info:
 
-        @solid(config={'bad_nested_list': [{'bad_field': 'kjdkfd'}]})
+        @solid(config_schema={'bad_nested_list': [{'bad_field': 'kjdkfd'}]})
         def _bad_list_config(_):
             pass
 
