@@ -18,11 +18,11 @@ from dagster import (
     OutputDefinition,
     PipelineDefinition,
     ScheduleDefinition,
+    composite_solid,
     execute_pipeline,
     execute_solid,
     lambda_solid,
     pipeline,
-    schedules,
     solid,
 )
 from dagster.core.definitions.decorators import (
@@ -274,13 +274,13 @@ def test_any_config_field():
     called = {}
     conf_value = 234
 
-    @solid(config=Field(Any))
+    @solid(config_schema=Field(Any))
     def hello_world(context):
         assert context.solid_config == conf_value
         called['yup'] = True
 
     result = execute_solid(
-        hello_world, environment_dict={'solids': {'hello_world': {'config': conf_value}}}
+        hello_world, run_config={'solids': {'hello_world': {'config': conf_value}}}
     )
 
     assert called['yup']
@@ -298,14 +298,13 @@ def test_solid_no_arg():
 
 
 def test_scheduler():
-    @schedules
-    def define_scheduler():
+    def define_schedules():
         return [
             ScheduleDefinition(
                 name="my_schedule",
                 cron_schedule="* * * * *",
                 pipeline_name="test_pipeline",
-                environment_dict={},
+                run_config={},
             )
         ]
 
@@ -454,10 +453,28 @@ def test_solid_docstring():
         '''QUUX_DOCSTRING'''
         return
 
+    @composite_solid
+    def comp_solid():
+        '''COMP_DOCSTRING'''
+        foo_solid()
+
+    @pipeline
+    def the_pipeline():
+        '''THE_DOCSTRING'''
+        quux_solid()
+
     assert foo_solid.__doc__ == 'FOO_DOCSTRING'
+    assert foo_solid.__name__ == 'foo_solid'
     assert bar_solid.__doc__ == 'BAR_DOCSTRING'
+    assert bar_solid.__name__ == 'bar_solid'
     assert baz_solid.__doc__ == 'BAZ_DOCSTRING'
+    assert baz_solid.__name__ == 'baz_solid'
     assert quux_solid.__doc__ == 'QUUX_DOCSTRING'
+    assert quux_solid.__name__ == 'quux_solid'
+    assert comp_solid.__doc__ == 'COMP_DOCSTRING'
+    assert comp_solid.__name__ == 'comp_solid'
+    assert the_pipeline.__doc__ == 'THE_DOCSTRING'
+    assert the_pipeline.__name__ == 'the_pipeline'
 
 
 def test_solid_yields_single_bare_value():
