@@ -1,25 +1,19 @@
+import re
+
 import pytest
 
 from dagster import (
-    ConfigType,
+    DagsterInvalidConfigDefinitionError,
     DagsterInvalidDefinitionError,
     DependencyDefinition,
-    Dict,
     Field,
     InputDefinition,
-    Int,
-    NamedDict,
     OutputDefinition,
-    PermissiveDict,
     PipelineDefinition,
     ResourceDefinition,
     SolidDefinition,
-    String,
     solid,
 )
-
-from dagster.core.definitions import create_environment_schema
-from dagster.core.types import Selector, NamedSelector
 from dagster.core.utility_solids import define_stub_solid
 
 
@@ -51,7 +45,7 @@ def test_create_pipeline_with_bad_solids_list():
 
 
 def test_circular_dep():
-    with pytest.raises(DagsterInvalidDefinitionError, match='Circular reference'):
+    with pytest.raises(DagsterInvalidDefinitionError, match='circular reference'):
         PipelineDefinition(
             solid_defs=solid_a_b_list(),
             dependencies={'A': {}, 'B': {'b_input': DependencyDefinition('B')}},
@@ -60,7 +54,7 @@ def test_circular_dep():
 
 def test_from_solid_not_there():
     with pytest.raises(
-        DagsterInvalidDefinitionError, match='Solid NOTTHERE in dependency dictionary not found'
+        DagsterInvalidDefinitionError, match='solid "NOTTHERE" in dependency dictionary not found'
     ):
         PipelineDefinition(
             solid_defs=solid_a_b_list(),
@@ -74,7 +68,7 @@ def test_from_solid_not_there():
 
 def test_from_non_existant_input():
     with pytest.raises(
-        DagsterInvalidDefinitionError, match='Solid "B" does not have input "not_an_input"'
+        DagsterInvalidDefinitionError, match='solid "B" does not have input "not_an_input"'
     ):
         PipelineDefinition(
             solid_defs=solid_a_b_list(),
@@ -84,8 +78,7 @@ def test_from_non_existant_input():
 
 def test_to_solid_not_there():
     with pytest.raises(
-        DagsterInvalidDefinitionError,
-        match='Solid NOTTHERE in DependencyDefinition not found in solid list',
+        DagsterInvalidDefinitionError, match='solid "NOTTHERE" not found in solid list'
     ):
         PipelineDefinition(
             solid_defs=solid_a_b_list(),
@@ -95,7 +88,7 @@ def test_to_solid_not_there():
 
 def test_to_solid_output_not_there():
     with pytest.raises(
-        DagsterInvalidDefinitionError, match='Solid A does not have output NOTTHERE'
+        DagsterInvalidDefinitionError, match='solid "A" does not have output "NOTTHERE"'
     ):
         PipelineDefinition(
             solid_defs=solid_a_b_list(),
@@ -138,101 +131,38 @@ def test_list_dependencies():
         PipelineDefinition(solid_defs=solid_a_b_list(), dependencies=[])
 
 
-def test_double_type_name():
-    @solid(config_field=Field(NamedDict('SomeTypeName', {'some_field': Field(String)})))
-    def solid_one(_context):
-        raise Exception('should not execute')
-
-    @solid(config_field=Field(NamedDict('SomeTypeName', {'another_field': Field(String)})))
-    def solid_two(_context):
-        raise Exception('should not execute')
-
-    with pytest.raises(DagsterInvalidDefinitionError) as exc_info:
-        create_environment_schema(PipelineDefinition(solid_defs=[solid_one, solid_two]))
-
-    assert str(exc_info.value) == (
-        'Type names must be unique. You have constructed two different instances of '
-        'types with the same name "SomeTypeName".'
-    )
-
-
-def test_double_type_key():
-    class KeyOneNameOneType(ConfigType):
-        def __init__(self):
-            super(KeyOneNameOneType, self).__init__(key='KeyOne', name='NameOne')
-
-    class KeyOneNameTwoType(ConfigType):
-        def __init__(self):
-            super(KeyOneNameTwoType, self).__init__(key='KeyOne', name='NameTwo')
-
-    @solid(config_field=Field(KeyOneNameOneType))
-    def solid_one(_context):
-        raise Exception('should not execute')
-
-    @solid(config_field=Field(KeyOneNameTwoType))
-    def solid_two(_context):
-        raise Exception('should not execute')
-
-    with pytest.raises(DagsterInvalidDefinitionError) as exc_info:
-        create_environment_schema(PipelineDefinition(solid_defs=[solid_one, solid_two]))
-
-    assert str(exc_info.value) == (
-        'Type keys must be unique. You have constructed two different instances of types '
-        'with the same key "KeyOne".'
-    )
-
-
-def test_pass_config_type_to_field_error_solid_definition():
-    with pytest.raises(DagsterInvalidDefinitionError) as exc_info:
-
-        @solid(config_field=Dict({'val': Field(Int)}))
-        def a_solid(_context):
-            pass
-
-        assert a_solid  # fool lint
-
-    assert str(exc_info.value) == (
-        'You have passed a config type "{ val: Int }" in the parameter "config_field" '
-        'of a SolidDefinition or @solid named "a_solid". You have '
-        'likely forgot to wrap this type in a Field.'
-    )
-
-
 def test_pass_unrelated_type_to_field_error_solid_definition():
 
-    with pytest.raises(DagsterInvalidDefinitionError) as exc_info:
+    with pytest.raises(DagsterInvalidConfigDefinitionError) as exc_info:
 
-        @solid(config_field='nope')
-        def a_solid(_context):
+        @solid(config_schema='nope')
+        def _a_solid(_context):
             pass
 
-        assert a_solid  # fool lint
-
-    assert str(exc_info.value) == (
-        'You have passed an object \'nope\' of incorrect type "str" in the parameter '
-        '"config_field" of a SolidDefinition or @solid named "a_solid" where a Field '
-        'was expected.'
-    )
-
-
-def test_pass_config_type_to_field_error_resource_definition():
-    with pytest.raises(DagsterInvalidDefinitionError) as exc_info:
-        ResourceDefinition(resource_fn=lambda: None, config_field=Dict({'val': Field(Int)}))
-
-    assert str(exc_info.value) == (
-        'You have passed a config type "{ val: Int }" in the parameter "config_field" of a '
-        'ResourceDefinition or @resource. You have likely forgot to '
-        'wrap this type in a Field.'
+    assert str(exc_info.value).startswith(
+        "Error defining config. Original value passed: 'nope'. 'nope' cannot be resolved."
     )
 
 
 def test_pass_unrelated_type_to_field_error_resource_definition():
-    with pytest.raises(DagsterInvalidDefinitionError) as exc_info:
-        ResourceDefinition(resource_fn=lambda: None, config_field='wut')
+    with pytest.raises(DagsterInvalidConfigDefinitionError) as exc_info:
+        ResourceDefinition(resource_fn=lambda: None, config_schema='wut')
 
-    assert str(exc_info.value) == (
-        'You have passed an object \'wut\' of incorrect type "str" in the parameter '
-        '"config_field" of a ResourceDefinition or @resource where a Field was expected.'
+    assert str(exc_info.value).startswith(
+        "Error defining config. Original value passed: 'wut'. 'wut' cannot be resolved."
+    )
+
+
+def test_pass_unrelated_type_in_nested_field_error_resource_definition():
+    with pytest.raises(DagsterInvalidConfigDefinitionError) as exc_info:
+        ResourceDefinition(
+            resource_fn=lambda: None, config_schema={'field': {'nested_field': 'wut'}}
+        )
+    assert str(exc_info.value).startswith('Error')
+
+    assert str(exc_info.value).startswith(
+        "Error defining config. Original value passed: {'field': {'nested_field': 'wut'}}. "
+        "Error at stack path :field:nested_field. 'wut' cannot be resolved."
     )
 
 
@@ -242,65 +172,77 @@ def test_pass_incorrect_thing_to_field():
 
     assert str(exc_info.value) == (
         'Attempted to pass \'nope\' to a Field that expects a valid dagster type '
-        'usable in config (e.g. Dict, NamedDict, Int, String et al).'
+        'usable in config (e.g. Dict, Int, String et al).'
     )
 
 
-def test_invalid_dict_field():
-    with pytest.raises(DagsterInvalidDefinitionError) as exc_info:
-        Dict({'val': Int, 'another_val': Field(Int)})
+def test_bad_output_definition():
+    with pytest.raises(
+        DagsterInvalidDefinitionError,
+        match=re.escape(
+            'Invalid type: dagster_type must be DagsterType, a python scalar, or '
+            'a python type that has been marked usable as a dagster type via '
+            '@usable_dagster_type or make_python_type_usable_as_dagster_type: '
+            'got foo'
+        ),
+    ):
+        _output = OutputDefinition('foo')
 
-    assert str(exc_info.value) == (
-        'You have passed a config type "Int" in the parameter "fields" and it is '
-        'in the "val" entry of that dict. It is from a Dict with fields '
-        '[\'another_val\', \'val\']. You have likely '
-        'forgot to wrap this type in a Field.'
-    )
+    # Test the case where the object is not hashable
+    with pytest.raises(
+        DagsterInvalidDefinitionError,
+        match=re.escape(
+            'Invalid type: dagster_type must be DagsterType, a python scalar, or '
+            'a python type that has been marked usable as a dagster type via '
+            '@usable_dagster_type or make_python_type_usable_as_dagster_type: '
+            'got {\'foo\': \'bar\'}, which isn\'t hashable. '
+            'Did you pass an instance of a type instead of the type?'
+        ),
+    ):
+        _output = OutputDefinition({'foo': 'bar'})
+
+    # Test the case where the object throws in __nonzero__, e.g. pandas.DataFrame
+    class Exotic(object):
+        def __nonzero__(self):
+            raise ValueError('Love too break the core Python APIs in widely-used libraries')
+
+    with pytest.raises(
+        DagsterInvalidDefinitionError,
+        match=re.escape(
+            'Invalid type: dagster_type must be DagsterType, a python scalar, or '
+            'a python type that has been marked usable as a dagster type via '
+            '@usable_dagster_type or make_python_type_usable_as_dagster_type: '
+            'got <dagster_tests.core_tests.definitions_tests.test_definition_errors'
+        )
+        + '('  # py27
+        + re.escape('.test_bad_output_definition.<locals>')
+        + ')?'
+        + re.escape('.Exotic object'),
+    ):
+        _output = OutputDefinition(Exotic())
 
 
-def test_invalid_named_dict_field():
-    with pytest.raises(DagsterInvalidDefinitionError) as exc_info:
-        NamedDict('some_named_dict', {'val': Int, 'another_val': Field(Int)})
+def test_solid_tags():
+    @solid(tags={'good': {'ok': 'fine'}})
+    def _fine_tags(_):
+        pass
 
-    assert str(exc_info.value) == (
-        'You have passed a config type "Int" in the parameter "fields" and it is '
-        'in the "val" entry of that dict. It is from a NamedDict named '
-        '"some_named_dict" with fields [\'another_val\', \'val\']. You '
-        'have likely forgot to wrap this type in a Field.'
-    )
+    class X:
+        pass
 
+    with pytest.raises(
+        DagsterInvalidDefinitionError, match='Could not JSON encode value',
+    ):
 
-def test_invalid_permissive_dict_field():
+        @solid(tags={'bad': X()})
+        def _bad_tags(_):
+            pass
 
-    with pytest.raises(DagsterInvalidDefinitionError) as exc_info:
-        PermissiveDict({'val': Int, 'another_val': Field(Int)})
+    with pytest.raises(
+        DagsterInvalidDefinitionError,
+        match=r'JSON encoding "\[1, 2\]" of value "\(1, 2\)" is not equivalent to original value',
+    ):
 
-    assert str(exc_info.value) == (
-        'You have passed a config type "Int" in the parameter "fields" and it is '
-        'in the "val" entry of that dict. It is from a PermissiveDict with fields '
-        '[\'another_val\', \'val\']. You have likely '
-        'forgot to wrap this type in a Field.'
-    )
-
-
-def test_invalid_selector_field():
-    with pytest.raises(DagsterInvalidDefinitionError) as exc_info:
-        Selector({'val': Int})
-
-    assert str(exc_info.value) == (
-        'You have passed a config type "Int" in the parameter "fields" and it is '
-        'in the "val" entry of that dict. It is from a Selector with fields '
-        '[\'val\']. You have likely forgot to wrap this type in a Field.'
-    )
-
-
-def test_invalid_named_selector_field():
-    with pytest.raises(DagsterInvalidDefinitionError) as exc_info:
-        NamedSelector('some_selector', {'val': Int})
-
-    assert str(exc_info.value) == (
-        'You have passed a config type "Int" in the parameter "fields" and it is '
-        'in the "val" entry of that dict. It is from a NamedSelector named '
-        '"some_selector" with fields [\'val\']. You '
-        'have likely forgot to wrap this type in a Field.'
-    )
+        @solid(tags={'set_comes_back_as_dict': (1, 2)})
+        def _also_bad_tags(_):
+            pass
