@@ -4,6 +4,7 @@ import pytest
 
 from dagster import (
     Bool,
+    DagsterInstance,
     DagsterInvalidDefinitionError,
     DagsterInvariantViolationError,
     ModeDefinition,
@@ -12,13 +13,14 @@ from dagster import (
     check,
     execute_pipeline,
     lambda_solid,
+    pipeline,
     solid,
 )
 from dagster.utils import file_relative_path
 
 
 def test_presets():
-    @solid(config={'error': Bool})
+    @solid(config_schema={'error': Bool})
     def can_fail(context):
         if context.solid_config['error']:
             raise Exception('I did an error')
@@ -28,34 +30,34 @@ def test_presets():
     def always_fail():
         raise Exception('I always do this')
 
-    pipeline = PipelineDefinition(
+    pipe = PipelineDefinition(
         name='simple',
         solid_defs=[can_fail, always_fail],
         preset_defs=[
             PresetDefinition.from_files(
                 'passing',
                 environment_files=[file_relative_path(__file__, 'pass_env.yaml')],
-                solid_subset=['can_fail'],
+                solid_selection=['can_fail'],
             ),
             PresetDefinition.from_files(
                 'passing_overide_to_fail',
                 environment_files=[file_relative_path(__file__, 'pass_env.yaml')],
-                solid_subset=['can_fail'],
+                solid_selection=['can_fail'],
             ).with_additional_config({'solids': {'can_fail': {'config': {'error': True}}}}),
             PresetDefinition(
                 'passing_direct_dict',
-                environment_dict={'solids': {'can_fail': {'config': {'error': False}}}},
-                solid_subset=['can_fail'],
+                run_config={'solids': {'can_fail': {'config': {'error': False}}}},
+                solid_selection=['can_fail'],
             ),
             PresetDefinition.from_files(
                 'failing_1',
                 environment_files=[file_relative_path(__file__, 'fail_env.yaml')],
-                solid_subset=['can_fail'],
+                solid_selection=['can_fail'],
             ),
             PresetDefinition.from_files(
                 'failing_2', environment_files=[file_relative_path(__file__, 'pass_env.yaml')]
             ),
-            PresetDefinition('subset', solid_subset=['can_fail'],),
+            PresetDefinition('subset', solid_selection=['can_fail'],),
         ],
     )
 
@@ -70,44 +72,40 @@ def test_presets():
             environment_files=[file_relative_path(__file__, 'test_repository_definition.py')],
         )
 
-    assert execute_pipeline(pipeline, preset='passing').success
+    assert execute_pipeline(pipe, preset='passing').success
 
-    assert execute_pipeline(pipeline, preset='passing_direct_dict').success
-    assert execute_pipeline(pipeline, preset='failing_1', raise_on_error=False).success == False
+    assert execute_pipeline(pipe, preset='passing_direct_dict').success
+    assert execute_pipeline(pipe, preset='failing_1', raise_on_error=False).success == False
 
-    assert execute_pipeline(pipeline, preset='failing_2', raise_on_error=False).success == False
+    assert execute_pipeline(pipe, preset='failing_2', raise_on_error=False).success == False
 
     with pytest.raises(DagsterInvariantViolationError, match='Could not find preset'):
-        execute_pipeline(pipeline, preset='not_failing', raise_on_error=False)
+        execute_pipeline(pipe, preset='not_failing', raise_on_error=False)
 
     assert (
-        execute_pipeline(pipeline, preset='passing_overide_to_fail', raise_on_error=False).success
+        execute_pipeline(pipe, preset='passing_overide_to_fail', raise_on_error=False).success
         == False
     )
 
     assert execute_pipeline(
-        pipeline,
-        preset='passing',
-        environment_dict={'solids': {'can_fail': {'config': {'error': False}}}},
+        pipe, preset='passing', run_config={'solids': {'can_fail': {'config': {'error': False}}}},
     ).success
 
     with pytest.raises(
         check.CheckError,
         match=re.escape(
             'The environment set in preset \'passing\' does not agree with the environment passed '
-            'in the `environment_dict` argument.'
+            'in the `run_config` argument.'
         ),
     ):
         execute_pipeline(
-            pipeline,
+            pipe,
             preset='passing',
-            environment_dict={'solids': {'can_fail': {'config': {'error': True}}}},
+            run_config={'solids': {'can_fail': {'config': {'error': True}}}},
         )
 
     assert execute_pipeline(
-        pipeline,
-        preset='subset',
-        environment_dict={'solids': {'can_fail': {'config': {'error': False}}}},
+        pipe, preset='subset', run_config={'solids': {'can_fail': {'config': {'error': False}}}},
     ).success
 
 
@@ -159,7 +157,7 @@ other: 4
 final: "result"
 '''
     preset = PresetDefinition.from_yaml_strings('passing', [a, b, c])
-    assert preset.environment_dict == {
+    assert preset.run_config == {
         'foo': {'bar': 1, 'one': 'one'},
         'baz': 3,
         'other': 4,
@@ -172,14 +170,14 @@ final: "result"
 
     res = PresetDefinition.from_yaml_strings('empty')
     assert res == PresetDefinition(
-        name='empty', environment_dict={}, solid_subset=None, mode='default'
+        name='empty', run_config={}, solid_selection=None, mode='default'
     )
 
 
 def test_from_pkg_resources():
     good = ('dagster_tests.core_tests.definitions_tests', 'pass_env.yaml')
     res = PresetDefinition.from_pkg_resources('pass', [good])
-    assert res.environment_dict == {'solids': {'can_fail': {'config': {'error': False}}}}
+    assert res.run_config == {'solids': {'can_fail': {'config': {'error': False}}}}
 
     bad_defs = [
         ('dagster_tests.core_tests.definitions_tests', 'does_not_exist.yaml'),
@@ -192,3 +190,43 @@ def test_from_pkg_resources():
             DagsterInvariantViolationError, match='Encountered error attempting to parse yaml',
         ):
             PresetDefinition.from_pkg_resources('bad_def', [bad_def])
+
+
+def test_tags():
+    @lambda_solid
+    def a_solid():
+        return 'solid'
+
+    @pipeline(
+        tags={'pipeline_tag': 'pipeline_tag', 'all': 'pipeline', 'defs': 'pipeline'},
+        preset_defs=[
+            PresetDefinition(
+                name='a_preset',
+                tags={'preset_tag': 'preset_tag', 'all': 'preset', 'defs': 'preset'},
+            )
+        ],
+    )
+    def a_pipeline():
+        a_solid()
+
+    instance = DagsterInstance.ephemeral()
+    result = execute_pipeline(
+        a_pipeline,
+        instance=instance,
+        preset='a_preset',
+        tags={'execute_tag': 'execute_tag', 'all': 'execute'},
+    )
+    assert result.success
+    pipeline_run = instance.get_run_by_id(result.run_id)
+
+    assert 'pipeline_tag' in pipeline_run.tags
+    assert 'preset_tag' in pipeline_run.tags
+    assert 'execute_tag' in pipeline_run.tags
+
+    # execute overwrites preset overwrites pipeline def
+    assert 'all' in pipeline_run.tags
+    assert pipeline_run.tags['all'] == 'execute'
+
+    # preset overwrites pipeline def
+    assert 'defs' in pipeline_run.tags
+    assert pipeline_run.tags['defs'] == 'preset'
