@@ -4,6 +4,7 @@ import pytest
 
 from dagster import (
     DagsterEventType,
+    DagsterInvalidDefinitionError,
     DagsterInvariantViolationError,
     DagsterTypeCheckDidNotPass,
     EventMetadataEntry,
@@ -18,6 +19,7 @@ from dagster import (
     check_dagster_type,
     execute_pipeline,
     lambda_solid,
+    make_python_type_usable_as_dagster_type,
     pipeline,
     resource,
     solid,
@@ -387,9 +389,7 @@ def test_fan_in_custom_types_with_storage():
         # Fan-in
         get_foo([return_dict_1(), return_dict_2()])
 
-    pipeline_result = execute_pipeline(
-        dict_pipeline, environment_dict={'storage': {'filesystem': {}}}
-    )
+    pipeline_result = execute_pipeline(dict_pipeline, run_config={'storage': {'filesystem': {}}})
     assert pipeline_result.success
 
 
@@ -602,3 +602,17 @@ def test_type_equality():
     assert not (
         resolve_dagster_type(Optional[List[int]]) != resolve_dagster_type(Optional[List[int]])
     )
+
+
+def test_make_usable_as_dagster_type_called_twice():
+    class AType:
+        pass
+
+    ADagsterType = PythonObjectDagsterType(AType, name='ADagsterType',)
+    BDagsterType = PythonObjectDagsterType(AType, name='BDagsterType',)
+
+    make_python_type_usable_as_dagster_type(AType, ADagsterType)
+    make_python_type_usable_as_dagster_type(AType, ADagsterType)  # should not raise an error
+
+    with pytest.raises(DagsterInvalidDefinitionError):
+        make_python_type_usable_as_dagster_type(AType, BDagsterType)
