@@ -5,16 +5,18 @@ from dagster import (
     DagsterExecutionStepNotFoundError,
     DagsterStepOutputNotFoundError,
     DependencyDefinition,
-    ExecutionTargetHandle,
     InputDefinition,
     Int,
     OutputDefinition,
     PipelineDefinition,
     lambda_solid,
+    reconstructable,
 )
 from dagster.core.execution.api import create_execution_plan, execute_plan
+from dagster.core.execution.plan.objects import StepOutputHandle
 from dagster.core.instance import DagsterInstance
 from dagster.core.storage.intermediate_store import build_fs_intermediate_store
+from dagster.core.storage.intermediates_manager import IntermediateStoreIntermediatesManager
 
 
 def define_inty_pipeline():
@@ -52,10 +54,10 @@ def get_step_output(step_events, step_key, output_name='result'):
 def test_using_file_system_for_subplan():
     pipeline = define_inty_pipeline()
 
-    environment_dict = {'storage': {'filesystem': {}}}
+    run_config = {'storage': {'filesystem': {}}}
 
     instance = DagsterInstance.ephemeral()
-    execution_plan = create_execution_plan(pipeline, environment_dict=environment_dict,)
+    execution_plan = create_execution_plan(pipeline, run_config=run_config,)
     pipeline_run = instance.create_run_for_pipeline(
         pipeline_def=pipeline, execution_plan=execution_plan
     )
@@ -65,42 +67,49 @@ def test_using_file_system_for_subplan():
         execute_plan(
             execution_plan.build_subset_plan(['return_one.compute']),
             instance,
-            environment_dict=environment_dict,
+            run_config=run_config,
             pipeline_run=pipeline_run,
         )
     )
 
     store = build_fs_intermediate_store(instance.intermediates_directory, pipeline_run.run_id)
+    intermediates_manager = IntermediateStoreIntermediatesManager(store)
     assert get_step_output(return_one_step_events, 'return_one.compute')
-    assert store.has_intermediate(None, 'return_one.compute')
-    assert store.get_intermediate(None, 'return_one.compute', Int).obj == 1
+    assert intermediates_manager.has_intermediate(None, StepOutputHandle('return_one.compute'))
+    assert (
+        intermediates_manager.get_intermediate(
+            None, Int, StepOutputHandle('return_one.compute')
+        ).obj
+        == 1
+    )
 
     add_one_step_events = list(
         execute_plan(
             execution_plan.build_subset_plan(['add_one.compute']),
             instance,
-            environment_dict=environment_dict,
+            run_config=run_config,
             pipeline_run=pipeline_run,
         )
     )
 
     assert get_step_output(add_one_step_events, 'add_one.compute')
-    assert store.has_intermediate(None, 'add_one.compute')
-    assert store.get_intermediate(None, 'add_one.compute', Int).obj == 2
+    assert intermediates_manager.has_intermediate(None, StepOutputHandle('add_one.compute'))
+    assert (
+        intermediates_manager.get_intermediate(None, Int, StepOutputHandle('add_one.compute')).obj
+        == 2
+    )
 
 
 def test_using_file_system_for_subplan_multiprocessing():
 
-    environment_dict = {'storage': {'filesystem': {}}}
+    run_config = {'storage': {'filesystem': {}}}
     instance = DagsterInstance.local_temp()
 
-    pipeline_def = ExecutionTargetHandle.for_pipeline_fn(
-        define_inty_pipeline
-    ).build_pipeline_definition()
+    pipeline = reconstructable(define_inty_pipeline)
 
-    execution_plan = create_execution_plan(pipeline_def, environment_dict=environment_dict)
+    execution_plan = create_execution_plan(pipeline, run_config=run_config)
     pipeline_run = instance.create_run_for_pipeline(
-        pipeline_def=pipeline_def, execution_plan=execution_plan
+        pipeline_def=pipeline.get_definition(), execution_plan=execution_plan
     )
 
     assert execution_plan.get_step_by_key('return_one.compute')
@@ -109,29 +118,38 @@ def test_using_file_system_for_subplan_multiprocessing():
         execute_plan(
             execution_plan.build_subset_plan(['return_one.compute']),
             instance,
-            environment_dict=dict(environment_dict, execution={'multiprocess': {}}),
+            run_config=dict(run_config, execution={'multiprocess': {}}),
             pipeline_run=pipeline_run,
         )
     )
 
     store = build_fs_intermediate_store(instance.intermediates_directory, pipeline_run.run_id)
+    intermediates_manager = IntermediateStoreIntermediatesManager(store)
 
     assert get_step_output(return_one_step_events, 'return_one.compute')
-    assert store.has_intermediate(None, 'return_one.compute')
-    assert store.get_intermediate(None, 'return_one.compute', Int).obj == 1
+    assert intermediates_manager.has_intermediate(None, StepOutputHandle('return_one.compute'))
+    assert (
+        intermediates_manager.get_intermediate(
+            None, Int, StepOutputHandle('return_one.compute')
+        ).obj
+        == 1
+    )
 
     add_one_step_events = list(
         execute_plan(
             execution_plan.build_subset_plan(['add_one.compute']),
             instance,
-            environment_dict=dict(environment_dict, execution={'multiprocess': {}}),
+            run_config=dict(run_config, execution={'multiprocess': {}}),
             pipeline_run=pipeline_run,
         )
     )
 
     assert get_step_output(add_one_step_events, 'add_one.compute')
-    assert store.has_intermediate(None, 'add_one.compute')
-    assert store.get_intermediate(None, 'add_one.compute', Int).obj == 2
+    assert intermediates_manager.has_intermediate(None, StepOutputHandle('add_one.compute'))
+    assert (
+        intermediates_manager.get_intermediate(None, Int, StepOutputHandle('add_one.compute')).obj
+        == 2
+    )
 
 
 def test_execute_step_wrong_step_key():
@@ -166,10 +184,10 @@ def test_execute_step_wrong_step_key():
 
 def test_using_file_system_for_subplan_missing_input():
     pipeline = define_inty_pipeline()
-    environment_dict = {'storage': {'filesystem': {}}}
+    run_config = {'storage': {'filesystem': {}}}
 
     instance = DagsterInstance.ephemeral()
-    execution_plan = create_execution_plan(pipeline, environment_dict=environment_dict)
+    execution_plan = create_execution_plan(pipeline, run_config=run_config)
     pipeline_run = instance.create_run_for_pipeline(
         pipeline_def=pipeline, execution_plan=execution_plan
     )
@@ -178,7 +196,7 @@ def test_using_file_system_for_subplan_missing_input():
         execute_plan(
             execution_plan.build_subset_plan(['add_one.compute']),
             instance,
-            environment_dict=environment_dict,
+            run_config=run_config,
             pipeline_run=pipeline_run,
         )
 
@@ -186,10 +204,10 @@ def test_using_file_system_for_subplan_missing_input():
 def test_using_file_system_for_subplan_invalid_step():
     pipeline = define_inty_pipeline()
 
-    environment_dict = {'storage': {'filesystem': {}}}
+    run_config = {'storage': {'filesystem': {}}}
 
     instance = DagsterInstance.ephemeral()
-    execution_plan = create_execution_plan(pipeline, environment_dict=environment_dict)
+    execution_plan = create_execution_plan(pipeline, run_config=run_config)
     pipeline_run = instance.create_run_for_pipeline(
         pipeline_def=pipeline, execution_plan=execution_plan
     )
@@ -198,6 +216,6 @@ def test_using_file_system_for_subplan_invalid_step():
         execute_plan(
             execution_plan.build_subset_plan(['nope']),
             instance,
-            environment_dict=environment_dict,
+            run_config=run_config,
             pipeline_run=pipeline_run,
         )
