@@ -3,23 +3,21 @@ import logging
 import pytest
 
 from dagster import (
+    DagsterInvalidConfigError,
     DagsterInvalidDefinitionError,
     DagsterInvariantViolationError,
-    Dict,
-    execute_pipeline,
     Field,
-    logger,
     ModeDefinition,
-    DagsterInvalidConfigError,
     PipelineDefinition,
-    resource,
-    RunConfig,
-    solid,
     String,
+    execute_pipeline,
+    logger,
+    pipeline,
+    resource,
+    solid,
 )
-from dagster.utils.test import execute_solids_within_pipeline
-from dagster.core.definitions.environment_schema import create_environment_type
 from dagster.core.log_manager import coerce_valid_log_level
+from dagster.utils.test import execute_solids_within_pipeline
 
 from ..test_repository import (
     define_multi_mode_pipeline,
@@ -40,6 +38,20 @@ def test_mode_takes_a_name():
     assert pipeline_def
 
 
+def test_mode_from_resources():
+    @solid(required_resource_keys={'three'})
+    def ret_three(context):
+        return context.resources.three
+
+    @pipeline(
+        name='takes a mode', mode_defs=[ModeDefinition.from_resources({'three': 3}, name='three')]
+    )
+    def pipeline_def():
+        return ret_three()
+
+    assert execute_pipeline(pipeline_def).result_for_solid('ret_three').output_value() == 3
+
+
 def test_execute_single_mode():
     single_mode_pipeline = define_single_mode_pipeline()
     assert single_mode_pipeline.is_single_mode is True
@@ -47,7 +59,7 @@ def test_execute_single_mode():
     assert execute_pipeline(single_mode_pipeline).result_for_solid('return_two').output_value() == 2
 
     assert (
-        execute_pipeline(single_mode_pipeline, run_config=RunConfig(mode='the_mode'))
+        execute_pipeline(single_mode_pipeline, mode='the_mode')
         .result_for_solid('return_two')
         .output_value()
         == 2
@@ -57,7 +69,7 @@ def test_execute_single_mode():
 def test_wrong_single_mode():
     with pytest.raises(DagsterInvariantViolationError):
         assert (
-            execute_pipeline(define_single_mode_pipeline(), run_config=RunConfig(mode='wrong_mode'))
+            execute_pipeline(pipeline=define_single_mode_pipeline(), mode='wrong_mode')
             .result_for_solid('return_two')
             .output_value()
             == 2
@@ -94,14 +106,14 @@ def test_execute_multi_mode():
     multi_mode_pipeline = define_multi_mode_pipeline()
 
     assert (
-        execute_pipeline(multi_mode_pipeline, run_config=RunConfig(mode='mode_one'))
+        execute_pipeline(pipeline=multi_mode_pipeline, mode='mode_one')
         .result_for_solid('return_three')
         .output_value()
         == 3
     )
 
     assert (
-        execute_pipeline(multi_mode_pipeline, run_config=RunConfig(mode='mode_two'))
+        execute_pipeline(pipeline=multi_mode_pipeline, mode='mode_two')
         .result_for_solid('return_three')
         .output_value()
         == 3
@@ -115,60 +127,25 @@ def test_execute_multi_mode_errors():
         execute_pipeline(multi_mode_pipeline)
 
     with pytest.raises(DagsterInvariantViolationError):
-        execute_pipeline(multi_mode_pipeline, run_config=RunConfig(mode='wrong_mode'))
+        execute_pipeline(pipeline=multi_mode_pipeline, mode='wrong_mode')
 
 
 def test_execute_multi_mode_with_resources():
     pipeline_def = define_multi_mode_with_resources_pipeline()
 
     add_mode_result = execute_pipeline(
-        pipeline_def,
-        run_config=RunConfig(mode='add_mode'),
-        environment_dict={'resources': {'op': {'config': 2}}},
+        pipeline=pipeline_def, mode='add_mode', run_config={'resources': {'op': {'config': 2}}},
     )
 
     assert add_mode_result.result_for_solid('apply_to_three').output_value() == 5
 
     mult_mode_result = execute_pipeline(
-        pipeline_def,
-        run_config=RunConfig(mode='mult_mode'),
-        environment_dict={'resources': {'op': {'config': 3}}},
+        pipeline=pipeline_def, mode='mult_mode', run_config={'resources': {'op': {'config': 3}}},
     )
 
     assert mult_mode_result.result_for_solid('apply_to_three').output_value() == 9
 
 
-def test_correct_env_type_names_for_named():
-    pipeline_def = define_multi_mode_with_resources_pipeline()
-
-    mult_type_name = create_environment_type(pipeline_def, 'mult_mode')
-    assert mult_type_name.key == 'MultiModeWithResources.Mode.MultMode.Environment'
-    assert mult_type_name.name == 'MultiModeWithResources.Mode.MultMode.Environment'
-
-    assert (
-        mult_type_name.fields['resources'].config_type.key
-        == 'MultiModeWithResources.Mode.MultMode.Resources'
-    )
-    assert (
-        mult_type_name.fields['resources'].config_type.name
-        == 'MultiModeWithResources.Mode.MultMode.Resources'
-    )
-
-    add_type_name = create_environment_type(pipeline_def, 'add_mode')
-
-    assert add_type_name.key == 'MultiModeWithResources.Mode.AddMode.Environment'
-    assert add_type_name.name == 'MultiModeWithResources.Mode.AddMode.Environment'
-
-    assert (
-        add_type_name.fields['resources'].config_type.key
-        == 'MultiModeWithResources.Mode.AddMode.Resources'
-    )
-    assert (
-        add_type_name.fields['resources'].config_type.name
-        == 'MultiModeWithResources.Mode.AddMode.Resources'
-    )
-
-
 def test_mode_with_resource_deps():
 
     called = {'count': 0}
@@ -201,10 +178,10 @@ def test_mode_with_resource_deps():
 
     assert (
         str(ide.value)
-        == 'Resource "a" is required by solid requires_a, but is not provided by mode "default".'
+        == 'Resource "a" is required by solid def requires_a, but is not provided by mode "default".'
     )
 
-    @solid
+    @solid(required_resource_keys={'a'})
     def no_deps(context):
         called['count'] += 1
         assert context.resources.a == 1
@@ -253,7 +230,7 @@ def test_subset_with_mode_definitions():
     assert called == {'a': 1, 'b': 1}
 
     assert (
-        execute_solids_within_pipeline(pipeline_def, solid_names=['requires_a'])[
+        execute_solids_within_pipeline(pipeline_def, solid_names={'requires_a'})[
             'requires_a'
         ].success
         is True
@@ -266,22 +243,14 @@ def define_multi_mode_with_loggers_pipeline():
     foo_logger_captured_results = []
     bar_logger_captured_results = []
 
-    @logger(
-        config_field=Field(
-            Dict({'log_level': Field(String, is_optional=True, default_value='INFO')})
-        )
-    )
+    @logger(config_schema={'log_level': Field(String, is_required=False, default_value='INFO')})
     def foo_logger(init_context):
         logger_ = logging.Logger('foo')
         logger_.log = lambda level, msg, **kwargs: foo_logger_captured_results.append((level, msg))
         logger_.setLevel(coerce_valid_log_level(init_context.logger_config['log_level']))
         return logger_
 
-    @logger(
-        config_field=Field(
-            Dict({'log_level': Field(String, is_optional=True, default_value='INFO')})
-        )
-    )
+    @logger(config_schema={'log_level': Field(String, is_required=False, default_value='INFO')})
     def bar_logger(init_context):
         logger_ = logging.Logger('bar')
         logger_.log = lambda level, msg, **kwargs: bar_logger_captured_results.append((level, msg))
@@ -310,36 +279,34 @@ def define_multi_mode_with_loggers_pipeline():
 
 
 def parse_captured_results(captured_results):
-    parsed_captured_results = [
-        [y.strip().split(' = ') for y in x[1].strip().split('\n')] for x in captured_results
-    ]
-    # the captured results are a list of lists of two-member lists [log_msg_key, log_msg_value]
-    # this double list comprehension grabs the log_msg_value where log_msg_key == 'orig_message'
-    original_messages = [
-        item[1]
-        for result in parsed_captured_results
-        for item in result
-        if item[0] == 'orig_message'
-    ]
-    return original_messages
+    # each result will be a tuple like:
+    # (10,
+    # 'multi_mode - 1cc8958b-5ce6-401a-9e2a-ddd653e59a7e - PIPELINE_START - Started execution of '
+    # 'pipeline "multi_mode".'
+    # )
+    # so, this will reconstruct the original message from the captured log line.
+
+    # Extract the text string and remove key = value tuples on later lines
+    return [x[1].split('\n')[0] for x in captured_results]
 
 
 def test_execute_multi_mode_loggers_with_single_logger():
-    pipeline_def, foo_logger_captured_results, bar_logger_captured_results = (
-        define_multi_mode_with_loggers_pipeline()
-    )
+    (
+        pipeline_def,
+        foo_logger_captured_results,
+        bar_logger_captured_results,
+    ) = define_multi_mode_with_loggers_pipeline()
 
     execute_pipeline(
-        pipeline_def,
-        run_config=RunConfig(mode='foo_mode'),
-        environment_dict={'loggers': {'foo': {'config': {'log_level': 'DEBUG'}}}},
+        pipeline=pipeline_def,
+        mode='foo_mode',
+        run_config={'loggers': {'foo': {'config': {'log_level': 'DEBUG'}}}},
     )
 
     assert not bar_logger_captured_results
 
     original_messages = parse_captured_results(foo_logger_captured_results)
-
-    assert len(list(filter(lambda x: x == '"Here we are"', original_messages))) == 1
+    assert len([x for x in original_messages if 'Here we are' in x]) == 1
 
 
 def test_execute_multi_mode_loggers_with_single_logger_extra_config():
@@ -347,9 +314,9 @@ def test_execute_multi_mode_loggers_with_single_logger_extra_config():
 
     with pytest.raises(DagsterInvalidConfigError):
         execute_pipeline(
-            pipeline_def,
-            run_config=RunConfig(mode='foo_mode'),
-            environment_dict={
+            pipeline=pipeline_def,
+            mode='foo_mode',
+            run_config={
                 'loggers': {
                     'foo': {'config': {'log_level': 'DEBUG'}},
                     'bar': {'config': {'log_level': 'DEBUG'}},
@@ -359,14 +326,16 @@ def test_execute_multi_mode_loggers_with_single_logger_extra_config():
 
 
 def test_execute_multi_mode_loggers_with_multiple_loggers():
-    pipeline_def, foo_logger_captured_results, bar_logger_captured_results = (
-        define_multi_mode_with_loggers_pipeline()
-    )
+    (
+        pipeline_def,
+        foo_logger_captured_results,
+        bar_logger_captured_results,
+    ) = define_multi_mode_with_loggers_pipeline()
 
     execute_pipeline(
-        pipeline_def,
-        run_config=RunConfig(mode='foo_bar_mode'),
-        environment_dict={
+        pipeline=pipeline_def,
+        mode='foo_bar_mode',
+        run_config={
             'loggers': {
                 'foo': {'config': {'log_level': 'DEBUG'}},
                 'bar': {'config': {'log_level': 'DEBUG'}},
@@ -376,26 +345,28 @@ def test_execute_multi_mode_loggers_with_multiple_loggers():
 
     foo_original_messages = parse_captured_results(foo_logger_captured_results)
 
-    assert len(list(filter(lambda x: x == '"Here we are"', foo_original_messages))) == 1
+    assert len([x for x in foo_original_messages if 'Here we are' in x]) == 1
 
     bar_original_messages = parse_captured_results(bar_logger_captured_results)
 
-    assert len(list(filter(lambda x: x == '"Here we are"', bar_original_messages))) == 1
+    assert len([x for x in bar_original_messages if 'Here we are' in x]) == 1
 
 
 def test_execute_multi_mode_loggers_with_multiple_loggers_single_config():
-    pipeline_def, foo_logger_captured_results, bar_logger_captured_results = (
-        define_multi_mode_with_loggers_pipeline()
-    )
+    (
+        pipeline_def,
+        foo_logger_captured_results,
+        bar_logger_captured_results,
+    ) = define_multi_mode_with_loggers_pipeline()
 
     execute_pipeline(
         pipeline_def,
-        run_config=RunConfig(mode='foo_bar_mode'),
-        environment_dict={'loggers': {'foo': {'config': {'log_level': 'DEBUG'}}}},
+        mode='foo_bar_mode',
+        run_config={'loggers': {'foo': {'config': {'log_level': 'DEBUG'}}}},
     )
 
     foo_original_messages = parse_captured_results(foo_logger_captured_results)
 
-    assert len(list(filter(lambda x: x == '"Here we are"', foo_original_messages))) == 1
+    assert len([x for x in foo_original_messages if 'Here we are' in x]) == 1
 
     assert not bar_logger_captured_results
