@@ -1,5 +1,11 @@
-import dagster.check as check
+import re
+import uuid
+
+import pytest
+
 from dagster import (
+    DagsterExecutionStepNotFoundError,
+    DagsterInvariantViolationError,
     DependencyDefinition,
     InputDefinition,
     Int,
@@ -12,20 +18,21 @@ from dagster import (
     OutputDefinition,
     PipelineDefinition,
     ResourceDefinition,
-    RunConfig,
     String,
+    check,
     execute_pipeline,
     execute_pipeline_iterator,
     lambda_solid,
     pipeline,
+    reexecute_pipeline,
     solid,
 )
 from dagster.core.definitions import Solid
-from dagster.core.definitions.container import _create_adjacency_lists
 from dagster.core.definitions.dependency import DependencyStructure
-from dagster.core.execution.api import step_output_event_filter
+from dagster.core.definitions.solid_container import _create_adjacency_lists
 from dagster.core.execution.results import SolidExecutionResult
 from dagster.core.instance import DagsterInstance
+from dagster.core.test_utils import step_output_event_filter
 from dagster.core.utility_solids import (
     create_root_solid,
     create_solid_with_deps,
@@ -322,7 +329,7 @@ def test_pipeline_subset():
     env_config = {'solids': {'add_one': {'inputs': {'num': {'value': 3}}}}}
 
     subset_result = execute_pipeline(
-        pipeline_def.build_sub_pipeline(['add_one']), environment_dict=env_config
+        pipeline_def.get_pipeline_subset_def({'add_one'}), run_config=env_config
     )
 
     assert subset_result.success
@@ -330,6 +337,71 @@ def test_pipeline_subset():
     assert subset_result.result_for_solid('add_one').output_value() == 4
 
 
+def test_pipeline_explicit_subset():
+    @lambda_solid
+    def return_one():
+        return 1
+
+    @lambda_solid
+    def add_one(num):
+        return num + 1
+
+    pipeline_def = PipelineDefinition(
+        solid_defs=[return_one, add_one],
+        dependencies={'add_one': {'num': DependencyDefinition('return_one')}},
+    )
+
+    pipeline_result = execute_pipeline(pipeline_def)
+    assert pipeline_result.success
+    assert pipeline_result.result_for_solid('add_one').output_value() == 2
+
+    env_config = {'solids': {'add_one': {'inputs': {'num': {'value': 3}}}}}
+
+    subset_result = execute_pipeline(
+        pipeline_def, run_config=env_config, solid_selection=['add_one']
+    )
+
+    assert subset_result.success
+    assert len(subset_result.solid_result_list) == 1
+    assert subset_result.result_for_solid('add_one').output_value() == 4
+
+
+def test_pipeline_subset_of_subset():
+    @lambda_solid
+    def return_one():
+        return 1
+
+    @lambda_solid
+    def add_one(num):
+        return num + 1
+
+    @pipeline
+    def pipeline_def():
+        add_one.alias('add_one_a')(return_one.alias('return_one_a')())
+        add_one.alias('add_one_b')(return_one.alias('return_one_b')())
+
+    pipeline_result = execute_pipeline(pipeline_def)
+    assert pipeline_result.success
+    assert len(pipeline_result.solid_result_list) == 4
+    assert pipeline_result.result_for_solid('add_one_a').output_value() == 2
+
+    subset_pipeline = pipeline_def.get_pipeline_subset_def({'add_one_a', 'return_one_a'})
+    subset_result = execute_pipeline(subset_pipeline)
+    assert subset_result.success
+    assert len(subset_result.solid_result_list) == 2
+    assert subset_result.result_for_solid('add_one_a').output_value() == 2
+
+    with pytest.raises(
+        DagsterInvariantViolationError, match='Pipeline subsets may not be subset again.'
+    ):
+        subset_pipeline.get_pipeline_subset_def({'add_one_a'})
+
+    with pytest.raises(
+        DagsterInvariantViolationError, match='Pipeline subsets may not be subset again.'
+    ):
+        subset_pipeline.get_pipeline_subset_def({'add_one_a', 'return_one_a'})
+
+
 def test_pipeline_subset_with_multi_dependency():
     @lambda_solid
     def return_one():
@@ -358,14 +430,57 @@ def test_pipeline_subset_with_multi_dependency():
     assert pipeline_result.success
     assert pipeline_result.result_for_solid('noop').output_value() == 3
 
-    subset_result = execute_pipeline(pipeline_def.build_sub_pipeline(['noop']))
+    subset_result = execute_pipeline(pipeline_def.get_pipeline_subset_def({'noop'}))
 
     assert subset_result.success
     assert len(subset_result.solid_result_list) == 1
     assert pipeline_result.result_for_solid('noop').output_value() == 3
 
     subset_result = execute_pipeline(
-        pipeline_def.build_sub_pipeline(['return_one', 'return_two', 'noop'])
+        pipeline_def.get_pipeline_subset_def({'return_one', 'return_two', 'noop'})
+    )
+
+    assert subset_result.success
+    assert len(subset_result.solid_result_list) == 3
+    assert pipeline_result.result_for_solid('noop').output_value() == 3
+
+
+def test_pipeline_explicit_subset_with_multi_dependency():
+    @lambda_solid
+    def return_one():
+        return 1
+
+    @lambda_solid
+    def return_two():
+        return 2
+
+    @lambda_solid(input_defs=[InputDefinition('dep', Nothing)])
+    def noop():
+        return 3
+
+    pipeline_def = PipelineDefinition(
+        solid_defs=[return_one, return_two, noop],
+        dependencies={
+            'noop': {
+                'dep': MultiDependencyDefinition(
+                    [DependencyDefinition('return_one'), DependencyDefinition('return_two')]
+                )
+            }
+        },
+    )
+
+    pipeline_result = execute_pipeline(pipeline_def)
+    assert pipeline_result.success
+    assert pipeline_result.result_for_solid('noop').output_value() == 3
+
+    subset_result = execute_pipeline(pipeline_def, solid_selection=['noop'])
+
+    assert subset_result.success
+    assert len(subset_result.solid_result_list) == 1
+    assert pipeline_result.result_for_solid('noop').output_value() == 3
+
+    subset_result = execute_pipeline(
+        pipeline_def, solid_selection=['return_one', 'return_two', 'noop']
     )
 
     assert subset_result.success
@@ -390,15 +505,17 @@ def define_three_part_pipeline():
 
 
 def define_created_disjoint_three_part_pipeline():
-    return define_three_part_pipeline().build_sub_pipeline(['add_one', 'add_three'])
+    return define_three_part_pipeline().get_pipeline_subset_def({'add_one', 'add_three'})
 
 
 def test_pipeline_disjoint_subset():
-    disjoint_pipeline = define_three_part_pipeline().build_sub_pipeline(['add_one', 'add_three'])
+    disjoint_pipeline = define_three_part_pipeline().get_pipeline_subset_def(
+        {'add_one', 'add_three'}
+    )
     assert len(disjoint_pipeline.solids) == 2
 
 
-def test_pipeline_execution_disjoint_subset():
+def test_pipeline_execution_explicit_disjoint_subset():
     env_config = {
         'solids': {
             'add_one': {'inputs': {'num': {'value': 2}}},
@@ -410,7 +527,7 @@ def test_pipeline_execution_disjoint_subset():
     pipeline_def = define_created_disjoint_three_part_pipeline()
 
     result = execute_pipeline(
-        pipeline_def.build_sub_pipeline(['add_one', 'add_three']), environment_dict=env_config
+        pipeline_def, solid_selection=['add_one', 'add_three'], run_config=env_config
     )
 
     assert result.success
@@ -439,24 +556,21 @@ def test_pipeline_wrapping_types():
 
     assert execute_pipeline(
         wrapping_test,
-        environment_dict={'solids': {'double_string_for_all': {'inputs': {'value': None}}}},
+        run_config={'solids': {'double_string_for_all': {'inputs': {'value': None}}}},
+    ).success
+
+    assert execute_pipeline(
+        wrapping_test, run_config={'solids': {'double_string_for_all': {'inputs': {'value': []}}}},
     ).success
 
     assert execute_pipeline(
         wrapping_test,
-        environment_dict={'solids': {'double_string_for_all': {'inputs': {'value': []}}}},
+        run_config={'solids': {'double_string_for_all': {'inputs': {'value': [{'value': 'foo'}]}}}},
     ).success
 
     assert execute_pipeline(
         wrapping_test,
-        environment_dict={
-            'solids': {'double_string_for_all': {'inputs': {'value': [{'value': 'foo'}]}}}
-        },
-    ).success
-
-    assert execute_pipeline(
-        wrapping_test,
-        environment_dict={
+        run_config={
             'solids': {'double_string_for_all': {'inputs': {'value': [{'value': 'bar'}, None]}}}
         },
     ).success
@@ -542,7 +656,7 @@ def test_pipeline_init_failure():
         stub_solid()
 
     result = execute_pipeline(
-        failing_init_pipeline, environment_dict=dict(env_config), raise_on_error=False
+        failing_init_pipeline, run_config=dict(env_config), raise_on_error=False
     )
 
     assert result.success is False
@@ -551,7 +665,7 @@ def test_pipeline_init_failure():
     assert event.pipeline_init_failure_data
 
 
-def test_reexecution():
+def test_reexecution_fs_storage():
     @lambda_solid
     def return_one():
         return 1
@@ -564,25 +678,122 @@ def test_reexecution():
         solid_defs=[return_one, add_one],
         dependencies={'add_one': {'num': DependencyDefinition('return_one')}},
     )
+    run_config = {'storage': {'filesystem': {}}}
     instance = DagsterInstance.ephemeral()
     pipeline_result = execute_pipeline(
-        pipeline_def, environment_dict={'storage': {'filesystem': {}}}, instance=instance
+        pipeline_def, run_config={'storage': {'filesystem': {}}}, instance=instance
     )
     assert pipeline_result.success
     assert pipeline_result.result_for_solid('add_one').output_value() == 2
 
-    reexecution_run_config = RunConfig(previous_run_id=pipeline_result.run_id)
-    reexecution_result = execute_pipeline(
-        pipeline_def,
-        environment_dict={'storage': {'filesystem': {}}},
-        run_config=reexecution_run_config,
-        instance=instance,
+    reexecution_result = reexecute_pipeline(
+        pipeline_def, pipeline_result.run_id, run_config=run_config, instance=instance,
     )
 
     assert reexecution_result.success
     assert len(reexecution_result.solid_result_list) == 2
     assert reexecution_result.result_for_solid('return_one').output_value() == 1
     assert reexecution_result.result_for_solid('add_one').output_value() == 2
+    reexecution_run = instance.get_run_by_id(reexecution_result.run_id)
+    assert reexecution_run.parent_run_id == pipeline_result.run_id
+    assert reexecution_run.root_run_id == pipeline_result.run_id
+
+    grandchild_result = reexecute_pipeline(
+        pipeline_def, reexecution_result.run_id, run_config=run_config, instance=instance,
+    )
+
+    assert grandchild_result.success
+    assert len(grandchild_result.solid_result_list) == 2
+    assert grandchild_result.result_for_solid('return_one').output_value() == 1
+    assert grandchild_result.result_for_solid('add_one').output_value() == 2
+    grandchild_run = instance.get_run_by_id(grandchild_result.run_id)
+    assert grandchild_run.parent_run_id == reexecution_result.run_id
+    assert grandchild_run.root_run_id == pipeline_result.run_id
+
+
+def test_reexecution_fs_storage_with_solid_selection():
+    @lambda_solid
+    def return_one():
+        return 1
+
+    @lambda_solid
+    def add_one(num):
+        return num + 1
+
+    pipeline_def = PipelineDefinition(
+        solid_defs=[return_one, add_one],
+        dependencies={'add_one': {'num': DependencyDefinition('return_one')}},
+    )
+    run_config = {'storage': {'filesystem': {}}}
+    instance = DagsterInstance.ephemeral()
+    # Case 1: re-execute a part of a pipeline when the original pipeline doesn't have solid selection
+    pipeline_result = execute_pipeline(pipeline_def, run_config, instance=instance)
+    assert pipeline_result.success
+    assert pipeline_result.result_for_solid('add_one').output_value() == 2
+
+    # This is how this is actually done in dagster_graphql.implementation.pipeline_execution_manager
+    reexecution_result_no_solid_selection = reexecute_pipeline(
+        pipeline_def,
+        parent_run_id=pipeline_result.run_id,
+        run_config=run_config,
+        step_keys_to_execute=['return_one.compute'],
+        instance=instance,
+    )
+    assert reexecution_result_no_solid_selection.success
+    assert len(reexecution_result_no_solid_selection.solid_result_list) == 2
+    assert reexecution_result_no_solid_selection.result_for_solid('add_one').skipped
+    assert reexecution_result_no_solid_selection.result_for_solid('return_one').output_value() == 1
+
+    # Case 2: re-execute a pipeline when the original pipeline has solid selection
+    pipeline_result_solid_selection = execute_pipeline(
+        pipeline_def, run_config=run_config, instance=instance, solid_selection=['return_one'],
+    )
+    assert pipeline_result_solid_selection.success
+    assert len(pipeline_result_solid_selection.solid_result_list) == 1
+    with pytest.raises(DagsterInvariantViolationError):
+        pipeline_result_solid_selection.result_for_solid('add_one')
+    assert pipeline_result_solid_selection.result_for_solid('return_one').output_value() == 1
+
+    reexecution_result_solid_selection = reexecute_pipeline(
+        pipeline_def,
+        parent_run_id=pipeline_result_solid_selection.run_id,
+        run_config=run_config,
+        instance=instance,
+    )
+
+    assert reexecution_result_solid_selection.success
+    assert len(reexecution_result_solid_selection.solid_result_list) == 1
+    with pytest.raises(DagsterInvariantViolationError):
+        pipeline_result_solid_selection.result_for_solid('add_one')
+    assert reexecution_result_solid_selection.result_for_solid('return_one').output_value() == 1
+
+    # Case 3: re-execute a pipeline partially when the original pipeline has solid selection and
+    #   re-exeucte a step which hasn't been included in the original pipeline
+    with pytest.raises(
+        DagsterExecutionStepNotFoundError,
+        match=re.escape('Execution plan does not contain step: add_one.compute'),
+    ):
+        reexecute_pipeline(
+            pipeline_def,
+            parent_run_id=pipeline_result_solid_selection.run_id,
+            run_config=run_config,
+            step_keys_to_execute=['add_one.compute'],
+            instance=instance,
+        )
+
+    # Case 4: re-execute a pipeline partially when the original pipeline has solid selection and
+    #   re-exeucte a step which has been included in the original pipeline
+    re_reexecution_result = reexecute_pipeline(
+        pipeline_def,
+        parent_run_id=reexecution_result_solid_selection.run_id,
+        run_config=run_config,
+        instance=instance,
+        step_keys_to_execute=['return_one.compute'],
+    )
+
+    assert re_reexecution_result.success
+    assert len(re_reexecution_result.solid_result_list) == 1
+    assert re_reexecution_result.result_for_solid('return_one').output_value() == 1
 
 
 def test_single_step_reexecution():
@@ -598,20 +809,19 @@ def test_single_step_reexecution():
         solid_defs=[return_one, add_one],
         dependencies={'add_one': {'num': DependencyDefinition('return_one')}},
     )
+    run_config = {'storage': {'filesystem': {}}}
     instance = DagsterInstance.ephemeral()
-    pipeline_result = execute_pipeline(
-        pipeline_def, environment_dict={'storage': {'filesystem': {}}}, instance=instance
-    )
+    pipeline_result = execute_pipeline(pipeline_def, run_config, instance=instance)
     assert pipeline_result.success
     assert pipeline_result.result_for_solid('add_one').output_value() == 2
 
-    reexecution_result = execute_pipeline(
+    # This is how this is actually done in dagster_graphql.implementation.pipeline_execution_manager
+    reexecution_result = reexecute_pipeline(
         pipeline_def,
-        environment_dict={'storage': {'filesystem': {}}},
-        run_config=RunConfig(
-            previous_run_id=pipeline_result.run_id, step_keys_to_execute=['add_one.compute']
-        ),
+        parent_run_id=pipeline_result.run_id,
+        run_config=run_config,
         instance=instance,
+        step_keys_to_execute=['add_one.compute'],
     )
 
     assert reexecution_result.success
@@ -633,22 +843,18 @@ def test_two_step_reexecution():
         add_one(add_one(return_one()))
 
     instance = DagsterInstance.ephemeral()
-    pipeline_result = execute_pipeline(
-        two_step_reexec, environment_dict={'storage': {'filesystem': {}}}, instance=instance
-    )
+    run_config = {'storage': {'filesystem': {}}}
+    pipeline_result = execute_pipeline(two_step_reexec, run_config=run_config, instance=instance)
     assert pipeline_result.success
     assert pipeline_result.result_for_solid('add_one_2').output_value() == 3
 
-    reexecution_result = execute_pipeline(
+    reexecution_result = reexecute_pipeline(
         two_step_reexec,
-        environment_dict={'storage': {'filesystem': {}}},
-        run_config=RunConfig(
-            previous_run_id=pipeline_result.run_id,
-            step_keys_to_execute=['add_one.compute', 'add_one_2.compute'],
-        ),
+        parent_run_id=pipeline_result.run_id,
+        run_config=run_config,
         instance=instance,
+        step_keys_to_execute=['add_one.compute', 'add_one_2.compute'],
     )
-
     assert reexecution_result.success
     assert reexecution_result.result_for_solid('return_one').output_value() == None
     assert reexecution_result.result_for_solid('add_one_2').output_value() == 3
@@ -680,19 +886,6 @@ def test_optional():
     assert result_optional.skipped
 
 
-def test_basic_pipeline_selector():
-    @solid
-    def def_one(_):
-        pass
-
-    # dsl subsets the definitions appropriately
-    @pipeline
-    def pipe():
-        def_one()
-
-    assert pipe.selector.solid_subset == None
-
-
 def test_selector_with_partial_dependency_dict():
     executed = {}
 
@@ -714,7 +907,7 @@ def test_selector_with_partial_dependency_dict():
     assert set(executed.keys()) == {'one', 'two'}
 
 
-def test_selector_with_build_sub_pipeline():
+def test_selector_with_subset_for_execution():
     @solid
     def def_one(_):
         pass
@@ -729,4 +922,44 @@ def test_selector_with_build_sub_pipeline():
         def_one()
         def_two()
 
-    assert set(pipe.build_sub_pipeline(['def_two']).selector.solid_subset) == {'def_two'}
+    assert pipe.get_pipeline_subset_def({'def_two'}).solids_to_execute == {'def_two'}
+
+
+def test_default_run_id():
+    called = {}
+
+    @solid
+    def check_run_id(context):
+        called['yes'] = True
+        assert uuid.UUID(context.run_id)
+        called['run_id'] = context.run_id
+
+    pipeline_def = PipelineDefinition(solid_defs=[check_run_id])
+
+    result = execute_pipeline(pipeline_def)
+    assert result.run_id == called['run_id']
+    assert called['yes']
+
+
+def test_pipeline_tags():
+    called = {}
+
+    @solid
+    def check_tags(context):
+        assert context.get_tag('foo') == 'bar'
+        called['yup'] = True
+
+    pipeline_def_with_tags = PipelineDefinition(
+        name='injected_run_id', solid_defs=[check_tags], tags={'foo': 'bar'}
+    )
+    result = execute_pipeline(pipeline_def_with_tags)
+    assert result.success
+    assert called['yup']
+
+    called = {}
+    pipeline_def_with_override_tags = PipelineDefinition(
+        name='injected_run_id', solid_defs=[check_tags], tags={'foo': 'notbar'}
+    )
+    result = execute_pipeline(pipeline_def_with_override_tags, tags={'foo': 'bar'})
+    assert result.success
+    assert called['yup']
